
import numpy as np
import rft1d


eps         = np.finfo(float).eps


class LinearModel(object):
	def __init__(self, Y, X):
		Y               = np.asarray(Y, dtype=float)
		self.dim        = Y.ndim - 1            #dependent variable dimensionality (0 or 1)
		self.Y          = self._asmatrix(Y)     #stacked dependent variable (JxQ)
		self.X          = np.matrix(X)          #design matrix
		self.J          = self.X.shape[0]       #number of observations
		self.Q          = self.Y.shape[1]       #number of field nodes
		self.QT         = None                  #QR decomposition of design matrix
		self.eij        = None                  #residuals
		# self.contrasts  = contrasts             #list of contrast objects
		self._R         = None                  #residual forming matrix
		self._beta      = None                  #least-squares parameters
		self._rankR     = None                  #error degrees of freedom
		self._dfE       = None                  #error degrees of freedom (equivalent to _rankR)
		self._SSE       = None                  #sum-of-squares (error)
		self._MSE       = None                  #mean-squared error
		if self.dim==1:
			self.eij    = None
			self.fwhm   = None
			self.resels = None
		### labels:
		self.term_labels = None
		self.Fterms      = None

	def _asmatrix(self, Y):
		return np.matrix(Y).T if Y.ndim==1 else np.matrix(Y)

	def _rank(self, A, tol=None):
		'''
		This is a slight modification of np.linalg.matrix_rank.
		The tolerance performs poorly for some matrices
		Here the tolerance is boosted by a factor of ten for improved performance.
		'''
		M = np.asarray(A)
		S = np.linalg.svd(M, compute_uv=False)
		if tol is None:
			tol = 10 * S.max() * max(M.shape) * np.finfo(M.dtype).eps
		rank = sum(S > tol)
		return rank

	def fit(self, approx_residuals=None):
		Y,X,J           = self.Y, self.X, self.J
		Xi              = np.linalg.pinv(X)         #design matrix pseudoinverse
		self._beta      = Xi*Y                      #estimated parameters
		self._R         = np.eye(J) - X*Xi          #residual forming matrix
		self._rankR     = self._rank(self._R)
		self._SSE       = np.diag( Y.T * self._R * Y )
		self._dfE       = self._rankR
		if self._dfE > eps:
			self._MSE = self._SSE / self._dfE
		if approx_residuals is None:
			self.eij    = np.asarray(self.Y - X*self._beta)  #residuals
		else:
			C           = approx_residuals
			A           = X * C.T
			Ai          = np.linalg.pinv(A)
			beta        = Ai*Y
			self.eij    = np.asarray(Y - A*beta)  #approximate residuals
		if self.dim==1:
			self.fwhm   = rft1d.geom.estimate_fwhm(self.eij)            #smoothness
			self.resels = rft1d.geom.resel_counts(self.eij, self.fwhm, element_based=False) #resel
		self.QT         = np.linalg.qr(X)[0].T







import numpy as np
from matplotlib import pyplot
import spm1d



#(0) Load dataset:
dataset      = spm1d.data.uv1d.anova1rm.SpeedGRFcategoricalRM()
Y,A,SUBJ     = dataset.get_data()




#(1) Run ANOVA:
alpha        = 0.05
equal_var    = True
F            = spm1d.stats.anova1(Y, A, equal_var)
Frm          = spm1d.stats.anova1rm(Y, A, SUBJ, equal_var)
Fi           = F.inference(alpha)
Firm         = Frm.inference(alpha)




#(2) Plot:
pyplot.close('all')
pyplot.figure( figsize=(8, 3.5) )
ax0     = pyplot.axes( (0.1, 0.15, 0.35, 0.8) )
ax1     = pyplot.axes((0.55,0.15,0.35,0.8))
### plot mean subject trajectories:
ax0.plot(Y[A==0].T, 'b')
ax0.plot(Y[A==1].T, 'k')
ax0.plot(Y[A==2].T, 'r')
### plot SPM results:
Firm.plot(ax=ax1, color='r', facecolor=(0.8,0.3,0.3), label='Within-subjects analysis')
Fi.plot(ax=ax1, label='Between-subjects analysis')
ax1.legend(fontsize=8)
pyplot.show()










import numpy as np
from .. import _base



class AnimalDepression(_base.DatasetMANOVA1):
	'''
	NOTES:  chisq value and p value computed in MATLAB
	'''
	def _set_values(self):
		self.www  = 'http://www.pearsonhighered.com/assets/hip/gb/uploads/Mayers_IntroStatsSPSS_Ch14.pdf'
		A         = np.array([
		[36,80,50,73,48,67],
		[48,93,28,87,48,50],
		[61,53,44,80,87,67],
		[42,53,44,62,42,50],
		[55,87,48,87,42,56],
		[42,60,67,67,42,56],
		[48,60,67,40,36,50],
		[48,98,50,90,61,49],
		[53,67,44,60,61,60],
		[48,93,80,93,42,48],
		])
		yA     = A[:,[0,3]]  #dogs
		yB     = A[:,[1,4]]  #cats
		yC     = A[:,[2,5]]  #hamsters
		self.Y = np.vstack([yA,yB,yC])
		self.A = np.array([0]*10 + [1]*10 + [2]*10)
		self.z    = 23.8481     #computed in MATLAB using manova1
		self.df   = (1, 4)
		self.p    = 0.000085673  #computed in MATLAB using manova1




class Stevens2002(_base.DatasetMANOVA1):
	'''
	NOTE 1:  In the PDF file there's an error on page 5.
	The third response in the K2 group should be (6,7) and not (5,7).
	NOTE 2:  The X2 and p values reported in the PDF have rounding errors.
	'''
	def _set_values(self):
		self.www  = 'http://faculty.smu.edu/kyler/courses/7314/MANOVA.pdf'
		self.Y    = np.array([[2,3],[3,4],[5,4],[2,5],    [4,8],[5,6],[6,7],    [7,6],[8,7],[10,8],[9,5],[7,6]  ])
		self.A    = np.array([0]*4 + [1]*3 + [2]*5)
		# self.z    = 20.4987
		# self.p    = 0.0004271
		self.z    = 20.4983     #computed in MATLAB using manova1
		self.df   = (1, 4)
		self.p    = 0.00039807  #computed in MATLAB using manova1









import designs
import factors
import models

from ui import anova1,anova1rm
from ui import anova2,anova2nested,anova2rm,anova2onerm
from ui import anova3,anova3nested,anova3rm,anova3tworm,anova3onerm




import warnings
import numpy as np
from matplotlib import pyplot


from factors import Factor,FactorNested,FactorNested2,FactorNestedTwoWay #FactorRM,FactorSubject







class Contrasts(object):
	def __init__(self, C, term_labels):
		self.C           = np.asarray(C, dtype=float)     #contrast matrix
		self.term_labels = term_labels

	def plot(self, ax=None):
		ax  = pyplot.gca() if ax==None else ax
		ax.imshow(self.C, interpolation='nearest', cmap='gray', vmin=-1, vmax=1, aspect='auto')
		xskip   = self.C.shape[1] / 10 + 1
		yskip   = self.C.shape[0] / 10 + 1
		pyplot.setp(ax, xticks=range(0, self.C.shape[1], xskip), yticks=range(0, self.C.shape[0], yskip))




class DesignBuilder(object):
	def __init__(self, labels=[]):
		self.COLS    = []
		self.labels  = list(labels)
		self.n       = len(self.labels)   #number of main factors
		self.ncol    = 0  #number of main columns
		self.nTerms  = 0  #total number of model terms
		self.colD    = dict(zip(self.labels,  [None]*self.n))
		self.XD      = dict(zip(self.labels,  [None]*self.n))


	def add_main_columns(self, label, X):
		self.XD[label]   = X
		i0,n             = self.ncol, X.shape[1]
		self.colD[label] = np.arange(i0, i0+n)
		self.COLS.append( range(i0,i0+n) )
		self.ncol       += n
		self.nTerms     += 1

	def get_contrasts(self):
		C         = np.zeros( (self.nTerms, self.ncol) )
		for i,col in enumerate(self.COLS):
			C[i,col] = 1
		return Contrasts(C, self.labels)

	def get_design_matrix(self):
		X   = np.hstack([self.XD[label]  for label in self.labels])
		return X






class _Design(object):
	def _get_column_const(self):
		return np.matrix( np.ones(self.J) ).T


	def plot(self, ax=None, plot_contrasts=True, contrastnums=[0,1,2]):
		if plot_contrasts:
			ax0 = pyplot.axes([0.05,0.05,0.4,0.9])
		else:
			ax0 = pyplot.axes()
		# self.design.plot(ax=ax0)
		ax0.imshow(self.X, interpolation='nearest', cmap='gray', vmin=-1, vmax=1, aspect='auto')
		xskip   = self.X.shape[1] / 10 + 1
		yskip   = self.X.shape[0] / 10 + 1
		pyplot.setp(ax0, xticks=range(0, self.X.shape[1], xskip), yticks=range(0, self.J, yskip))
		### plot contrasts:
		if plot_contrasts and self.contrasts!=None:
			yy  = np.linspace(0.7, 0.05, 3)
			if len(self.contrasts) < 4:
				for i,contrast in enumerate(self.contrasts):
					ax  = pyplot.axes([0.53,yy[i],0.45,0.2])
					contrast.plot(ax=ax)
			else:
				for i,ci in enumerate(contrastnums):
					ax  = pyplot.axes([0.53,yy[i],0.45,0.2])
					self.contrasts[ci].plot(ax=ax)
		pyplot.show()





class ANOVA1(_Design):
	def __init__(self, A):
		self.X           = None       #design matrix
		self.A           = Factor(A)  #factor levels
		self.J           = self.A.J   #number of observations
		self.contrasts   = None       #contrast matrix
		self.term_labels = ['Intercept', 'A']
		self.f_terms     = [('A','Error')]
		self._assemble()
		
	

	def _assemble(self):
		### assemble design matrix columns:
		XCONST         = self._get_column_const()
		XA             = self.A.get_design_main()
		### specify builder and add design matrix columns:
		builder        = DesignBuilder(labels=self.term_labels)
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A', XA)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()




class ANOVA1rm(_Design):
	def __init__(self, A, SUBJ):
		self.X          = None          #design matrix
		self.S          = Factor(SUBJ)  #subjects
		self.A          = Factor(A)     #factor levels
		self.J          = self.A.J      #number of observations
		self.contrasts  = None          #contrast matrix
		self.term_labels = ['Intercept', 'A', 'S', 'SA']
		self.f_terms     = [('A','SA')]
		self._check_balanced()
		self._assemble()


	def _assemble(self):
		### assemble design matrix columns:
		XCONST         = self._get_column_const()
		XA             = self.A.get_design_main()
		XS             = self.S.get_design_main()
		XSA            = self.A.get_design_interaction(self.S)
		### specify builder and add design matrix columns:
		builder        = DesignBuilder(labels=self.term_labels)
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A', XA)
		builder.add_main_columns('S', XS)
		builder.add_main_columns('SA', XSA)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()

	def _check_balanced(self):
		if not (self.A.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced(self.A):
			raise( ValueError('Design must be balanced.') )







class ANOVA2(_Design):
	def __init__(self, A, B):
		self.X           = None       #design matrix
		self.A           = Factor(A)  #factor level vector
		self.B           = Factor(B)  #factor level vector
		self.J           = self.A.J   #number of observations
		self.contrasts   = None
		self.balanced    = True
		self.term_labels = ['Intercept', 'A', 'B', 'AB']
		self.f_terms     = [('A','Error'), ('B','Error'), ('AB','Error')]
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XCONST         = self._get_column_const()
		XA             = self.A.get_design_main()
		XB             = self.B.get_design_main()
		XAB            = self.A.get_design_interaction(self.B)
		### specify builder and add design matrix columns:
		builder          = DesignBuilder(labels=self.term_labels)
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A',  XA)
		builder.add_main_columns('B',  XB)
		builder.add_main_columns('AB', XAB)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced):
			self.balanced = False
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			self.balanced = False
			raise( ValueError('Design must be balanced.') )





class ANOVA2nested(ANOVA2):
	def __init__(self, A, B):
		self.X          = None
		self.A          = Factor(A)
		self.B          = FactorNested(B, self.A)
		self.J          = self.A.J
		self.contrasts  = None
		self.term_labels = ['Intercept', 'A', 'B']
		self.f_terms     = [('A','B'), ('B','Error')]
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XCONST         = self._get_column_const()
		XA             = self.A.get_design_main()
		XB             = self.B.get_design_main()
		# XB             = self.B.get_design_main_nested(self.A)
		### specify builder and add design matrix columns:
		builder          = DesignBuilder(labels=['Intercept', 'A', 'B'])
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A',  XA)
		builder.add_main_columns('B',  XB)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced_nested(self.B):
			raise( ValueError('Design must be balanced.') )



class ANOVA2rm(ANOVA2):
	'''Both A and B are RM factors.'''
	def __init__(self, A, B, SUBJ):
		self.X           = None
		self.S           = Factor(SUBJ)
		self.A           = Factor(A)
		self.B           = Factor(B)
		self.J           = self.A.J
		self.contrasts   = None
		self.term_labels = ['Intercept', 'A', 'B', 'S', 'AB', 'SA', 'SB', 'SAB']
		self.f_terms     = [('A','SA'), ('B','SB'), ('AB','SAB')]
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XCONST          = self._get_column_const()
		XA              = self.A.get_design_main()
		XB              = self.B.get_design_main()
		XS              = self.S.get_design_main()
		XAB             = self.A.get_design_interaction(self.B)
		XSA             = self.A.get_design_interaction(self.S)
		XSB             = self.B.get_design_interaction(self.S)
		XSAB            = self.A.get_design_interaction_3way(self.B, self.S)
		### specify builder and add design matrix columns:
		builder           = DesignBuilder(labels=self.term_labels)
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A', XA)
		builder.add_main_columns('B', XB)
		builder.add_main_columns('S', XS)
		builder.add_main_columns('AB', XAB)
		builder.add_main_columns('SA', XSA)
		builder.add_main_columns('SB', XSB)
		builder.add_main_columns('SAB', XSAB)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()


	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced(self.A):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )


	def check_for_single_responses(self):
		A,B,S  = self.A.A, self.B.A, self.S.A
		only_single = False
		for a in self.A.u:
			for b in self.B.u:
				s = S[(A==a) & (B==b)]
				if np.unique(s).size == s.size:
					only_single = True
					warnings.warn('\nWARNING:  Only one observation per subject found.  Residuals and inference will be approximate. To avoid approximate residuals: (a) Add multiple observations per subject and per condition, and (b) ensure that all subjects and conditions have the same number of observations.\n', UserWarning, stacklevel=2)
					continue
		return only_single



class ANOVA2onerm(ANOVA2rm):
	'''Only B is an RM factor.'''
	def __init__(self, A, B, SUBJ):
		self.X           = None
		self.A           = Factor(A)
		self.B           = Factor(B)
		self.S           = FactorNested(SUBJ, self.A)
		self.J           = self.A.J
		self.contrasts   = None
		self.term_labels = ['Intercept', 'A', 'B', 'S', 'AB', 'SB']
		self.f_terms     = [('A','S'), ('B','SB'), ('AB','SB')]
		# self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XCONST          = self._get_column_const()
		XA              = self.A.get_design_main()
		XB              = self.B.get_design_main()
		XS              = self.S.get_design_main()
		XAB             = self.A.get_design_interaction(self.B)
		XSB             = self.S.get_design_interaction(self.B)
		### specify builder and add design matrix columns:
		builder         = DesignBuilder(labels=self.term_labels)
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A', XA)
		builder.add_main_columns('B', XB)
		builder.add_main_columns('S', XS)
		builder.add_main_columns('AB', XAB)
		builder.add_main_columns('SB', XSB)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()


	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.A):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.B):
			raise( ValueError('Design must be balanced.') )










class ANOVA3(_Design):
	def __init__(self, A, B, C):
		self.X          = None       #design matrix
		self.A          = Factor(A)  #factor level vector
		self.B          = Factor(B)  #factor level vector
		self.C          = Factor(C)  #factor level vector
		self.J          = self.A.J   #number of observations
		self.term_labels = ['Intercept', 'A', 'B', 'C', 'AB', 'AC', 'BC', 'ABC']
		self.f_terms     = [('A','Error'), ('B','Error'), ('C','Error'), ('AB','Error'), ('AC','Error'), ('BC','Error'), ('ABC','Error')]
		self.contrasts  = None
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XCONST         = self._get_column_const()
		XA             = self.A.get_design_main()
		XB             = self.B.get_design_main()
		XC             = self.C.get_design_main()
		XAB            = self.A.get_design_interaction(self.B)
		XAC            = self.A.get_design_interaction(self.C)
		XBC            = self.B.get_design_interaction(self.C)
		XABC           = self.A.get_design_interaction_3way(self.B, self.C)
		### specify builder and add design matrix columns:
		builder        = DesignBuilder(labels=['Intercept', 'A', 'B', 'C', 'AB', 'AC', 'BC', 'ABC'])
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A',  XA)
		builder.add_main_columns('B',  XB)
		builder.add_main_columns('C',  XC)
		builder.add_main_columns('AB', XAB)
		builder.add_main_columns('AC', XAC)
		builder.add_main_columns('BC', XBC)
		builder.add_main_columns('ABC', XABC)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()


	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.C.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.B.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )





class ANOVA3nested(ANOVA3):
	def __init__(self, A, B, C):
		self.X          = None
		self.A          = Factor(A)
		self.B          = FactorNested(B, self.A)
		self.C          = FactorNested2(C, self.B)
		self.J          = self.A.J
		self.contrasts  = None
		self.term_labels = ['Intercept', 'A', 'B', 'C']
		self.f_terms     = [('A','B'), ('B','C'), ('C','Error')]
		self._check_balanced()
		self._assemble()

	def _assemble(self):
		### assemble design matrix columns:
		XCONST         = self._get_column_const()
		XA             = self.A.get_design_main()
		XB             = self.B.get_design_main()
		XC             = self.C.get_design_main()
		### specify builder and add design matrix columns:
		builder          = DesignBuilder(labels=['Intercept', 'A', 'B', 'C'])
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A',  XA)
		builder.add_main_columns('B',  XB)
		builder.add_main_columns('C',  XC)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.C.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced_nested3(self.B, self.C):
			raise( ValueError('Design must be balanced.') )



class ANOVA3rm(ANOVA3):
	'''A, B and C are all RM factors.'''
	def __init__(self, A, B, C, SUBJ):
		self.X          = None
		self.A           = Factor(A)
		self.B           = Factor(B)
		self.C           = Factor(C)
		self.S           = Factor(SUBJ)
		self.J           = self.A.J
		self.contrasts   = None
		self.term_labels = ['Intercept',  'A','B','C','S',  'AB','AC','BC',   'SA','SB','SC',   'SAB','SAC','SBC',  'ABC', 'SABC']
		self.f_terms     = [('A','SA'), ('B','SB'), ('C','SC'),  ('AB','SAB'), ('AC','SAC'), ('BC','SBC'), ('ABC','SABC')]
		self._check_balanced()
		self._assemble()


	def _assemble(self):
		### assemble design matrix columns:
		XCONST          = self._get_column_const()
		XA              = self.A.get_design_main()
		XB              = self.B.get_design_main()
		XC              = self.C.get_design_main()
		XS              = self.S.get_design_main()
		XAB             = self.A.get_design_interaction(self.B)
		XAC             = self.A.get_design_interaction(self.C)
		XBC             = self.B.get_design_interaction(self.C)
		XSA             = self.S.get_design_interaction(self.A)
		XSB             = self.S.get_design_interaction(self.B)
		XSC             = self.S.get_design_interaction(self.C)
		XSAB            = self.S.get_design_interaction_3way(self.A, self.B)
		XSAC            = self.S.get_design_interaction_3way(self.A, self.C)
		XSBC            = self.S.get_design_interaction_3way(self.B, self.C)
		XABC            = self.A.get_design_interaction_3way(self.B, self.C)
		XSABC           = self.S.get_design_interaction_4way(self.A, self.B, self.C)
		### specify builder and add design matrix columns:
		builder           = DesignBuilder(labels=self.term_labels)
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A', XA)
		builder.add_main_columns('B', XB)
		builder.add_main_columns('C', XC)
		builder.add_main_columns('S', XS)
		builder.add_main_columns('AB', XAB)
		builder.add_main_columns('AC', XAC)
		builder.add_main_columns('BC', XBC)
		builder.add_main_columns('SA', XSA)
		builder.add_main_columns('SB', XSB)
		builder.add_main_columns('SC', XSC)
		builder.add_main_columns('SAB', XSAB)
		builder.add_main_columns('SAC', XSAC)
		builder.add_main_columns('SBC', XSBC)
		builder.add_main_columns('ABC', XABC)
		builder.add_main_columns('SABC', XSABC)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()

	def _swapAB(self):
		if self._swap:
			A,B         = self.B, self.A
			self.A      = A
			self.B      = B

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.C.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.B.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.C):
			raise( ValueError('Design must be balanced.') )


	def check_for_single_responses(self):
		A,B,C,S  = self.A.A, self.B.A, self.C.A, self.S.A
		only_single = False
		for a in self.A.u:
			for b in self.B.u:
				for c in self.C.u:
					s = S[(A==a) & (B==b) & (C==c)]
					if np.unique(s).size == s.size:
						only_single = True
						warnings.warn('\nWARNING:  Only one observation per subject found.  Residuals and inference will be approximate. To avoid approximate residuals: (a) Add multiple observations per subject and per condition, and (b) ensure that all subjects and conditions have the same number of observations.\n', UserWarning, stacklevel=2)
						continue
		return only_single





class ANOVA3onerm(ANOVA3rm):
	'''Only C is an RM factor.'''
	def __init__(self, A, B, C, SUBJ):
		self.X           = None
		self.A           = Factor(A)
		self.B           = Factor(B)
		self.C           = Factor(C)
		self.S           = FactorNestedTwoWay(SUBJ, self.A, self.B)
		# self.S           = FactorNestedTwoWay(SUBJ, self.B, self.C)
		self.J           = self.A.J
		self.contrasts   = None
		self.term_labels = ['Intercept',  'A','B','C', 'AB','AC','BC',    'ABC', 'S', 'SC']
		self.f_terms     = [('A','S'), ('B','S'), ('C','SC'),  ('AB','S'), ('AC','SC'), ('BC','SC'), ('ABC','SC')]
		self._check_balanced()
		self._assemble()


	def _assemble(self):
		### assemble design matrix columns:
		XCONST          = self._get_column_const()
		XA              = self.A.get_design_main()
		XB              = self.B.get_design_main()
		XC              = self.C.get_design_main()
		XS              = self.S.get_design_main()
		XAB             = self.A.get_design_interaction(self.B)
		XAC             = self.A.get_design_interaction(self.C)
		XBC             = self.B.get_design_interaction(self.C)
		XABC            = self.C.get_design_interaction_3way(self.A, self.B)
		XSC             = self.S.get_design_interaction(self.C)
		### specify builder and add design matrix columns:
		builder           = DesignBuilder(labels=self.term_labels)
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A', XA)
		builder.add_main_columns('B', XB)
		builder.add_main_columns('C', XC)
		builder.add_main_columns('AB', XAB)
		builder.add_main_columns('AC', XAC)
		builder.add_main_columns('BC', XBC)
		builder.add_main_columns('ABC', XABC)
		builder.add_main_columns('S', XS)
		builder.add_main_columns('SC', XSC)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()

	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.C.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.B.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.C):
			raise( ValueError('Design must be balanced.') )




class ANOVA3tworm(ANOVA3rm):
	'''Both B and C are RM factors.'''
	def __init__(self, A, B, C, SUBJ):
		self.X          = None
		self.A           = Factor(A)
		self.B           = Factor(B)
		self.C           = Factor(C)
		self.S           = FactorNested(SUBJ, self.A)
		# self.S           = FactorNestedTwoWay(SUBJ, self.B, self.C)
		self.J           = self.A.J
		self.contrasts   = None
		self.term_labels = ['Intercept',  'A','B','C','S',  'AB','AC','BC',  'SB','SC',   'ABC', 'SBC']
		self.f_terms     = [('A','S'), ('B','SB'), ('C','SC'),  ('AB','SB'), ('AC','SC'), ('BC','SBC'), ('ABC','SBC')]
		self._check_balanced()
		self._assemble()


	def _assemble(self):
		### assemble design matrix columns:
		XCONST          = self._get_column_const()
		XA              = self.A.get_design_main()
		XB              = self.B.get_design_main()
		XC              = self.C.get_design_main()
		XS              = self.S.get_design_main()
		XAB             = self.A.get_design_interaction(self.B)
		XAC             = self.A.get_design_interaction(self.C)
		XBC             = self.B.get_design_interaction(self.C)
		XSB             = self.S.get_design_interaction(self.B)
		XSC             = self.S.get_design_interaction(self.C)
		XABC            = self.A.get_design_interaction_3way(self.B, self.C)
		XSBC            = self.S.get_design_interaction_3way(self.B, self.C)
		### specify builder and add design matrix columns:
		builder           = DesignBuilder(labels=self.term_labels)
		builder.add_main_columns('Intercept', XCONST)
		builder.add_main_columns('A', XA)
		builder.add_main_columns('B', XB)
		builder.add_main_columns('C', XC)
		builder.add_main_columns('S', XS)
		builder.add_main_columns('AB', XAB)
		builder.add_main_columns('AC', XAC)
		builder.add_main_columns('BC', XBC)
		builder.add_main_columns('SB', XSB)
		builder.add_main_columns('SC', XSC)
		builder.add_main_columns('ABC', XABC)
		builder.add_main_columns('SBC', XSBC)
		### assemble design matrix and contrasts:
		self.X         = builder.get_design_matrix()
		self.contrasts = builder.get_contrasts()



	def _check_balanced(self):
		if not (self.A.balanced and self.B.balanced and self.C.balanced and self.S.balanced):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.A.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.B.check_balanced(self.C):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.B):
			raise( ValueError('Design must be balanced.') )
		if not self.S.check_balanced_rm(self.C):
			raise( ValueError('Design must be balanced.') )









'''
This module contains a variety of plotting functions.

The following functions may be accessed as methods of **spm1d** SPM objects:

=========================  ===================== ===============================
spm1d.plot                 SPM instance method   SPM inference instance method
=========================  ===================== ===============================
plot_spm                   plot
plot_spm_design            plot_design           plot_design
plot_spmi                                        plot
plot_spmi_p_values                               plot_p_values
plot_spmi_threshold_label                        plot_threshold_label
=========================  ===================== ===============================


All other plotting functions can only be accessed via **spm1d.plot**.
These include:

- plot_cloud
- plot_errorcloud
- plot_mean_sd
'''

# Copyright (C) 2015  Todd Pataky
# plot.py version: 0.3.0005 (2015/12/09)


from math import pi,sin,cos,acos,atan2
import numpy as np
from scipy import ndimage
import matplotlib
from matplotlib import pyplot, cm as colormaps
from matplotlib.patches import Polygon
from matplotlib.collections import PatchCollection


eps    = np.finfo(float).eps   #smallest float, used to avoid divide-by-zero errors



def _gca(ax):
	if ax==None:
		ax = pyplot.gca()
	return ax
def _getQ(x, Q):
	if x==None:
		x    = np.arange(Q)
	return x
def _set_ylim(ax, pad=0.075):
	def minmax(x):
		return min(x), max(x)
	ymin,ymax   = +1e10, -1e10
	for line in ax.lines:
		y0,y1   = minmax( line.get_data()[1] )
		ymin    = min(y0, ymin)
		ymax    = max(y1, ymax)
	for collection in ax.collections:
		datalim = collection.get_datalim(ax.transData)
		y0,y1   = minmax(  np.asarray(datalim)[:,1]  )
		ymin    = min(y0, ymin)
		ymax    = max(y1, ymax)
	for text in ax.texts:
		r       = matplotlib.backend_bases.RendererBase()
		bbox    = text.get_window_extent(r)
		y0,y1   = ax.transData.inverted().transform(bbox)[:,1]
		ymin    = min(y0, ymin)
		ymax    = max(y1, ymax)
	dy = 0.075*(ymax-ymin)
	ax.set_ylim(ymin-dy, ymax+dy)
def _stat2str(STAT):
	if STAT=='T':
		s    = 't'
	else:
		s    = STAT
	return s
def p2string(p):
	return 'p < 0.001' if p<0.0005 else 'p = %.03f'%p
	






def plot_cloud(Y, ax=None, facecolor='0.8', edgecolor='0.8', alpha=0.5, autoset_ylim=True):
	'''
	Plot an arbitrary 1D cloud.
	
	:Parameters:
	
	- *Y* --- a (2 x Q) numpy array containing y coordinates of the cloud's top and bottom surfaces, respectively
	- *ax* --- optional matplotlib.axes object
	- *facecolor* --- optional face color (for the SD cloud)
	- *edgecolor* --- optional edge color (for the SD cloud)
	- *alpha* --- optional face alpha value (for the SD cloud)
	- *autoset_ylim* --- if True (default), will set the y axis limits so that all text, line and patch objects are visible inside the axes

	:Returns:
	
	- a **matplotlib.collections.PatchCollection** object
	
	:Example:
	
	>>> import numpy as np
	>>> from matplotlib import pyplot
	
	>>> y_top     = np.random.rand(50)
	>>> y_bottom  = -0.5 * np.ones(50)
	>>> Y         = np.vstack([y_top, y_bottom])
	>>> spm1d.plot.plot_cloud(Y)
	>>> pyplot.xlim(0, 50)
	>>> pyplot.ylim(-1, 1)
	'''
	x           = _getQ(None, Y.shape[1])
	ax          = _gca(ax)
	### create patches:
	y0,y1       = Y
	x,y0,y1     = x.tolist(), y0.tolist(), y1.tolist()
	x           = [x[0]]  + x  + [x[-1]]
	y0          = [y0[0]] + y0 + [y0[-1]]
	y1          = [y1[0]] + y1 + [y1[-1]]
	y1.reverse()
	### concatenate:
	x1          = np.copy(x).tolist()
	x1.reverse()
	x,y         = x + x1, y0 + y1
	patches     = PatchCollection([Polygon(zip(x,y))], edgecolors=None)
	### plot:
	ax.add_collection(patches)
	pyplot.setp(patches, facecolor=facecolor, edgecolor=edgecolor, alpha=alpha)
	if autoset_ylim:
		_set_ylim(ax)
	return patches






def plot_errorcloud(datum, sd, ax=None, facecolor='0.8', edgecolor='0.8', alpha=0.5, autoset_ylim=True, perpendicular=False):
	'''
	Plot an arbitrary error cloud surrounding a datum continuum.
	
	:Parameters:
	
	- *datum* --- a 1D list or numpy array
	- *sd* --- a 1D list or numpy array
	- *ax* --- optional matplotlib.axes object
	- *facecolor* --- optional face color (for the SD cloud)
	- *edgecolor* --- optional edge color (for the SD cloud)
	- *alpha* --- optional face alpha value (for the SD cloud)
	- *autoset_ylim* --- if True (default), will set the y axis limits so that all text, line and patch objects are visible inside the axes

	:Returns:
	
	- a **matplotlib.collections.PatchCollection** object
	
	:Example:
	
	>>> import numpy as np
	>>> from matplotlib import pyplot
	
	>>> a     = np.random.rand(50)
	>>> b     = np.random.rand(50)
	>>> spm1d.plot.plot_errorcloud(a, b)
	>>> pyplot.xlim(0, 50)
	'''
	ax       = _gca(ax)
	y,s      = np.asarray(datum, dtype=float), np.asarray(sd, dtype=float)
	Y        = np.array([y+s, y-s])
	h        = plot_cloud(Y, ax, facecolor, edgecolor, alpha, autoset_ylim=False)
	# if perpendicular:
	# 	w        = 0.2
	# 	x        = _getQ(None, y.size)
	# 	line     = ThickSegmentedLine(x, y, sd=s)
	# 	h        = line.plot(vertical_ends=True, lw=3)
	#
	# else:
	# 	Y        = np.array([y+s, y-s])
	# 	h        = plot_cloud(Y, ax, facecolor, edgecolor, alpha, autoset_ylim=False)
	if autoset_ylim:
		_set_ylim(ax)
	return h





def plot_filled(y, ax=None, thresh=None, plot_thresh=True, color='k', lw=2, facecolor='0.8', two_tailed=False, thresh_color='k', autoset_ylim=True, label=None):
	'''
	Plot a filled cluster.
	
	(This function should only be used through **spm1d.plot.plot_spmi**)
	'''
	y        = np.asarray(y, dtype=float)
	x        = _getQ(None, y.size)
	ax       = _gca(ax)
	if thresh==None:
		thresh      = 0
	x0,y0,ind0      = x.copy(), y.copy(), np.arange(y.size)
	### threshold:
	L,n       = ndimage.label(y>thresh)
	if two_tailed:
		L1,n1 = ndimage.label(y<-thresh)
		L1[L1>0] += n
		L    += L1
		n    += n1
		### relabel left-to-right:
		ind1 = np.argsort([(L==(i+1)).argmax()  for i in range(4)])
		LL   = np.zeros(L.shape)
		for i,ind in enumerate(ind1):
			LL[L==(ind+1)] = i+1
		L    = LL
	### plot:
	ax.plot(x0, y0, color=color, lw=lw, label=label)
	### create patches if needed:
	if n>0:
		polyg = []
		for i in range(n):
			ind         = ind0[L==i+1].tolist()
			x           = x0[L==i+1].tolist()
			y           = y0[L==i+1].tolist()
			csign       = np.sign(y[0])
			### insert extra nodes for interpolation:
			x           = [x[0]] + x + [x[-1]]
			y           = [csign*thresh] + y + [csign*thresh]
			### interpolate if necessary:
			if ind[0]  != ind0[0]:
				dx      = x0[ind[0]] - x0[ind[0]-1]
				dy      = (csign*thresh - y0[ind[0]])  / (y0[ind[0]] - y0[ind[0]-1])
				x[0]   += dy*dx
			if ind[-1] != ind0[-1]:
				dx      = x0[ind[-1]+1] - x0[ind[-1]]
				dy      = (csign*thresh - y0[ind[-1]])  / (y0[ind[-1]+1] - y0[ind[-1]])
				x[-1]  += dy*dx
			polyg.append(  Polygon(zip(x,y))  )
		patches         = PatchCollection(polyg, edgecolors=None)
		ax.add_collection(patches)
		pyplot.setp(patches, facecolor=facecolor, edgecolor=facecolor)
	#set axis limits:
	pyplot.setp(ax, xlim=(x0.min(), x0.max())  )
	#plot threshold(s):
	if (thresh!=None) and plot_thresh:
		h      = [ax.hlines(thresh, x0.min(), x0.max())]
		if two_tailed:
			h.append( ax.hlines(-thresh, x0.min(), x0.max()) )
		pyplot.setp(h, color=thresh_color, lw=1, linestyle='--')
	if autoset_ylim:
		_set_ylim(ax)






def plot_mean_sd(Y, ax=None, lw=3, linecolor='k', linestyle='-', facecolor='0.8', edgecolor='0.8', alpha=0.5, label=None, autoset_ylim=True, perpendicular=True):
	'''
	Plot mean continuum with standard deviation cloud.
	
	:Parameters:
	
	- *Y* --- a (J x Q) numpy array
	- *ax* --- optional matplotlib.axes object  [default: matplotlib.pyplot.gca()]
	- *lw* --- optional integer specify line width
	- *linecolor* --- optional line color specifier (for the mean continuum)
	- *linestyle* --- optional line style specifier (for the mean continuum)
	- *facecolor* --- optional face color (for the SD cloud)
	- *edgecolor* --- optional edge color (for the SD cloud)
	- *alpha* --- optional face alpha value (for the SD cloud)
	- *label* --- optional string to label the mean continuum (for use with matplotlib.pyplot.legend())
	- *autoset_ylim* --- if True (default), will set the y axis limits so that all text, line and patch objects are visible inside the axes
	
	:Returns:
	
	- *None*
	
	:Example:
	
	>>> Y  = np.random.randn(10,101)
	>>> spm1d.plot.plot_mean_sd(Y)
	'''
	x        = _getQ(None, Y.shape[1])
	ax       = _gca(ax)
	m,s      = Y.mean(axis=0), Y.std(ddof=1, axis=0)
	h        = ax.plot(x, m, color=linecolor, lw=lw, linestyle=linestyle)[0]
	if label:
		h.set_label(label)
	plot_errorcloud(m, s, ax, facecolor, edgecolor, alpha, autoset_ylim=False, perpendicular=perpendicular)
	pyplot.setp(ax, xlim=(x.min(), x.max())  )
	if autoset_ylim:
		_set_ylim(ax)






def plot_spm(spm, ax=None, plot_ylabel=True, autoset_ylim=True, **kwdargs):
	'''
	Plot an **spm1d** SPM object as a line.
	
	:Parameters:
	
	- *spm* --- an **spm1d** SPM object (not needed if using the SPM.plot method)
	- *ax* --- optional matplotlib.axes object  [default: matplotlib.pyplot.gca()]
	- *plot_ylabel* --- if *True*, then an "SPM{t}" or "SPM{F}" label will automatically be added to the y axis
	- *autoset_ylim* --- if True (default), will set the y axis limits so that all text, line and patch objects are visible inside the axes
	- *kwdards* --- any keyword argument accepted by **matplotlib.pyplot.plot**
	
	:Returns:
	
	- *h* --- a **matplotlib.lines.Line2D** object
	
	:Example:
	
	>>> t     = spm1d.stats.ttest(Y)
	>>> line  = t.plot()   # equivalent to "line = spm1d.plot.plot_spm(t)"
	>>> line.set_color('r')
	'''
	ax     = _gca(ax)
	x      = _getQ(None, spm.Q)
	keys   = kwdargs.keys()
	if 'color' not in keys:
		kwdargs.update( dict(color='k') )
	if ('lw' not in keys) or ('linewidth' not in keys):
		kwdargs.update( dict(lw=2) )
	if plot_ylabel:
		ax.set_ylabel('SPM{%s}'%_stat2str(spm.STAT), size=16)
	h      = ax.plot(x, spm.z, **kwdargs)[0]
	if autoset_ylim:
		_set_ylim(ax)
	return h




def plot_spm_design(spm, ax=None, factor_labels=None, fontsize=10):
	'''
	Plot the design matrix.
	
	:Returns:
	
	None
	'''
	def scaleColumns(X):
		mn,mx     = np.min(X,axis=0) , np.max(X,axis=0)
		Xs        = (X-mn)/(mx-mn+eps)
		Xs[np.isnan(Xs)] = 1   #if the whole column is a constant
		return Xs
	ax            = _gca(ax)
	X             = spm.X
	vmin,vmax     = None, None
	if np.all(X==1):
		vmin,vmax = 0, 1
	ax.imshow(scaleColumns(X), cmap=colormaps.gray, interpolation='nearest', vmin=vmin, vmax=vmax)
	if factor_labels != None:
		gs        = X.shape
		tx        = [ax.text(i, -0.05*gs[0], label)   for i,label in enumerate(factor_labels)]
		pyplot.setp(tx, ha='center', va='bottom', color='k', fontsize=fontsize)
	ax.axis('normal')
	ax.axis('off')






def plot_spmi(spmi, ax=None, color='k', facecolor='0.8', plot_thresh=True, plot_ylabel=True, thresh_color='k', autoset_ylim=True, label=None):
	'''
	Plot an **spm1d** SPM inference object as a line.
	
	:Parameters:
	
	- *spmi* --- an **spm1d** SPM object
	- *ax* --- optional matplotlib.axes object  [default: matplotlib.pyplot.gca()]
	- *color* --- optional line color specifier (for the raw SPM)
	- *facecolor* --- optional face color (for suprathreshold clusters)
	- *plot_thresh* --- if *True*, one or two horizontal threshold lines will be plotted (for one- or two-tailed inference)
	- *plot_ylabel* --- if *True*, an "SPM{t}" or "SPM{F}" label will automatically be added to the y axis
	- *autoset_ylim* --- if True (default), will set the y axis limits so that all text, line and patch objects are visible inside the axes
	
	:Returns:
	
	- *None*
	
	:Example:
	
	>>> t     = spm1d.stats.ttest(Y)
	>>> ti    = t.inference(0.05)
	>>> ti.plot()   # equivalent to "spm1d.plot.plot_spmi(ti)"
	'''
	ax     = _gca(ax)
	x      = _getQ(None, spmi.Q)
	plot_filled(spmi.z, ax, spmi.zstar, color=color, plot_thresh=plot_thresh, facecolor=facecolor, two_tailed=spmi.two_tailed, thresh_color=thresh_color, autoset_ylim=False, label=label)
	ax.hlines(0, x.min(), x.max(), color='k', lw=1, linestyle=':')
	if plot_ylabel:
		ax.set_ylabel('SPM{%s}'%_stat2str(spmi.STAT), size=16)
	if autoset_ylim:
		_set_ylim(ax)




def plot_spmi_p_values(spmi, ax=None, size=8, offsets=None, offset_all_clusters=None, autoset_ylim=True):
	'''
	Plot an **spm1d** SPM inference object's p values as text (if they exist).
	
	:Parameters:
	
	- *spmi* --- an **spm1d** SPM inference object
	- *ax* --- optional matplotlib.axes object  [default: matplotlib.pyplot.gca()]
	- *size* --- optional integer specifying font size
	- *offsets* --- optional list of 2-tuples specifying (x,y) offsets with respect to cluster centroids
	- *offset_all_clusters* --- optional 2-tuple specifying the (x,y) offset for all clusters, with respect to cluster centroids
	- *autoset_ylim* --- if True (default), will set the y axis limits so that all text, line and patch objects are visible inside the axes
	
	:Returns:
	
	- *None*
	
	:Example:
	
	>>> t   = spm1d.stats.ttest(Y)
	>>> ti  = t.inference(0.05)
	>>> ti.plot()
	>>> myoffsets = [(0,0), (0,0.2), (0,0.1)]  # if there are three clusters, there must be three 2-tuple offsets
	>>> ti.plot_p_values(offsets=myoffsets) #equivalent to: "spm1d.plot.plot_p_values(ti, offsets=myoffsets)"
	
	'''
	ax         = _gca(ax)
	n          = len(spmi.p)
	if offsets is None:
		if offset_all_clusters is None:
			offsets = [(0,0)]*n
		else:
			offsets = [offset_all_clusters]*n
	if len(offsets) < n:
		print('WARNING:  there are fewer offsets than clusters.  To set offsets for all clusters use the offset_all_clusters keyword.')
	for cluster,offset in zip(spmi.clusters, offsets):
		x,y    = cluster.xy
		x     += offset[0]
		y     += offset[1]
		s      = p2string(cluster.P)
		ax.text(x, y, s, size=size, ha='center', va='center', bbox=dict(facecolor='w', alpha=0.3))
	if autoset_ylim:
		_set_ylim(ax)






def plot_spmi_threshold_label(spmi, ax=None, lower=False, pos=None, autoset_ylim=True, **kwdargs):
	'''
	Plot an **spm1d** SPM inference object as a line.
	
	:Parameters:
	
	- *spmi* --- an **spm1d** SPM inference object
	- *ax* --- optional matplotlib.axes object  [default: matplotlib.pyplot.gca()]
	- *lower* --- if True, will plot the label on the lower threshold (if two-tailed inference has been conducted)
	- *pos* --- optional 2-tuple specifying text object location; setting "pos" over-rides "lower"
	- *autoset_ylim* --- if True (default), will set the y axis limits so that all text, line and patch objects are visible inside the axes
	- *kwdards* --- any keyword argument accepted by **matplotlib.pyplot.text**
	
	:Returns:
	
	- a **matplotlib.text.Text** object
	
	:Example:
	
	>>> t     = spm1d.stats.ttest(Y)
	>>> ti    = t.inference(0.05)
	>>> ti.plot_threshold_label(pos=(50,3.0))   # equivalent to "spm1d.plot.plot_spmi_threshold_label(ti, pos=(50,3.0))"
	'''
	q         = _getQ(None, spmi.Q)
	ax        = _gca(ax)
	if pos==None:
		x0,x1 = q.min(), q.max()
		y0,y1 = ax.get_ylim()
		x     = x0 + 0.4*(x1-x0)
		if lower and spmi.two_tailed:
			y     = -spmi.zstar + 0.005*(y1-y0)
		else:
			y     = spmi.zstar + 0.005*(y1-y0)
	else:
		x,y   = pos
	if 'color' not in kwdargs.keys():
		kwdargs.update( dict(color='r') )
	s         = r'$\alpha$=%.2f:  $%s^*$=%.3f' %(spmi.alpha, _stat2str(spmi.STAT), spmi.zstar)
	h         = ax.text(x, y, s, **kwdargs)
	if autoset_ylim:
		_set_ylim(ax)
	return h





import numpy as np
from matplotlib import pyplot
import spm1d



# compute within-speed means for each subject:
nSubj        = 10
Y1,Y2,Y3     = [], [], []
for subj in range(10):
	### load data:
	dataset  = spm1d.data.uv1d.anova1.SpeedGRFcategorical(subj=subj)
	Y,A      = dataset.get_data()
	### compute means for each of the three conditions:
	y1,y2,y3 = Y[A==1], Y[A==2], Y[A==3] 
	Y1.append(  y1.mean(axis=0)  )
	Y2.append(  y2.mean(axis=0)  )
	Y3.append(  y3.mean(axis=0)  )
Y1,Y2,Y3     = np.asarray(Y1), np.asarray(Y2), np.asarray(Y3)


# plot:
pyplot.close('all')
h1 = pyplot.plot(Y1.T, color='b')
h2 = pyplot.plot(Y2.T, color='k')
h3 = pyplot.plot(Y3.T, color='r')
h1[0].set_label('Slow')
h2[0].set_label('Normal')
h3[0].set_label('Fast')
pyplot.legend(loc='lower center', fontsize=12)
pyplot.xlabel('Time (% stance)')
pyplot.ylabel('GRF (BW)')
pyplot.show()





'''
spm1D: A Python package for 1D Statistical Parametric Mapping

Useful references include:

[1] Statistical Parametric Mapping: The Analysis of Functional Brain Images
Karl Friston, John Ashburner, Stefan Kiebel, Thomas Nichols (Editors)
Academic Press, 2006

[2] Wellcome Trust Centre for Neuroimaging SPM homepage:
www.fil.ion.ucl.ac.uk/spm/

[3] Pataky TC (2010). Generalized n-dimensional field analysis using
statistical parametric mapping. Journal of Biomechanics.


Copyright (C) 2015  Todd Pataky
Version: 0.3.1.5 (2015/12/09)
'''


__version__ = '0.3.1.5 (2015/12/09)'


__all__ = ['io', 'plot', 'stats', 'util']


import data, io, plot, stats, util







import os
import numpy as np
from .. import _base





class _BadNoResiduals(_base.DatasetANOVA2rm):
	def _set_values(self):
		Y  = np.array( [] )
		self.Y     = np.random.randn(12, 101)
		self.A     = np.array( [0,0,0, 0,0,0, 1,1,1, 1,1,1] )
		self.B     = np.array( [0,0,0, 1,1,1,  0,0,0,  1,1,1] )
		self.SUBJ  = np.array( [0,1,2, 0,1,2, 0,1,2, 0,1,2]  )
		self.expected_warning = True
		self.expected_class = UserWarning
		
		


class _SPM1D_ANOVA2RM_DATASET(_base.DatasetANOVA2rm, _base.Dataset1D):
	def _set_values(self):
		self._set_datafile()
		Z             = np.load(self.datafile)
		self.Y,self.A,self.B,self.SUBJ = Z['Y'], Z['A'], Z['B'], Z['SUBJ']
		Z.close()


class SPM1D_ANOVA2RM_2x2(_SPM1D_ANOVA2RM_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2rm_2x2.npz')
class SPM1D_ANOVA2RM_2x3(_SPM1D_ANOVA2RM_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2rm_2x3.npz')

class SPM1D_ANOVA2RM_3x3(_SPM1D_ANOVA2RM_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2rm_3x3.npz')
class SPM1D_ANOVA2RM_3x4(_SPM1D_ANOVA2RM_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2rm_3x4.npz')
class SPM1D_ANOVA2RM_3x5(_SPM1D_ANOVA2RM_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2rm_3x5.npz')

class SPM1D_ANOVA2RM_4x4(_SPM1D_ANOVA2RM_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2rm_4x4.npz')
class SPM1D_ANOVA2RM_4x5(_SPM1D_ANOVA2RM_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2rm_4x5.npz')


# coding: utf-8

import os
import numpy as np
from .. import _base


class Besier2009kneeflexion(_base.DatasetANOVA2, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Besier, T. F., Fredericson, M., Gold, G. E., Beaupré, G. S., & Delp, S. L. (2009). Knee muscle forces during walking and running in patellofemoral pain patients and pain-free controls. Journal of Biomechanics, 42(7), 898–905. http://doi.org/10.1016/j.jbiomech.2009.01.032'
		self.www      = 'https://simtk.org/home/muscleforces'
		self.datafile = os.path.join(_base.get_datafilepath(), 'Besier2009kneeflexion.npz')
		self.note     = 'Results  ', 'Pataky, T. C., Vanrenterghem, J., & Robinson, M. A. (2015). Two-way ANOVA for scalar trajectories, with experimental evidence of non-phasic interactions. Journal of Biomechanics, 48(1), 186–189. http://doi.org/10.1016/j.jbiomech.2014.10.013'
		### load and parse data:
		Z             = np.load(self.datafile)
		self.Y        = Z['Y']
		self.A        = Z['GROUP']   #0:controls, 1:PFP
		self.B        = Z['GENDER']  #0:female, 1:male
		Z.close()
		self.z        = None
		self.df       = None
		self.p        = None



class Dorn2012(_base.DatasetANOVA2, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Dorn, T. W., Schache, A. G., & Pandy, M. G. (2012). Muscular strategy shift in human running: dependence of running speed on hip and ankle muscle performance. Journal of Experimental Biology, 215(11), 1944–1956. http://doi.org/10.1242/jeb.064527'
		self.www      = 'https://simtk.org/home/runningspeeds'
		self.datafile = os.path.join(_base.get_datafilepath(), 'Dorn2012.npz')
		### load and parse data:
		Z             = np.load(self.datafile)
		Y,A,B0        = Z['Y'], Z['FOOT'], Z['SPEED']
		Z.close()
		### choose one component:
		Y             = Y[:,:,1]
		B,uB          = np.zeros(B0.size), np.unique(B0)
		for i,u in enumerate(uB):
			B[B0==u]  = i
		B             = np.asarray(B, dtype=int)
		self.Y        = Y
		self.A        = A
		self.B        = B
		self.z        = None
		self.df       = None
		self.p        = None


class _SPM1D_ANOVA2_DATASET(_base.DatasetANOVA2, _base.Dataset1D):
	def _set_values(self):
		self._set_datafile()
		Z             = np.load(self.datafile)
		self.Y,self.A,self.B = Z['Y'], Z['A'], Z['B']
		Z.close()


class SPM1D_ANOVA2_2x2(_SPM1D_ANOVA2_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2_2x2.npz')
class SPM1D_ANOVA2_2x3(_SPM1D_ANOVA2_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2_2x3.npz')

class SPM1D_ANOVA2_3x3(_SPM1D_ANOVA2_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2_3x3.npz')
class SPM1D_ANOVA2_3x4(_SPM1D_ANOVA2_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2_3x4.npz')
class SPM1D_ANOVA2_3x5(_SPM1D_ANOVA2_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2_3x5.npz')

class SPM1D_ANOVA2_4x4(_SPM1D_ANOVA2_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2_4x4.npz')
class SPM1D_ANOVA2_4x5(_SPM1D_ANOVA2_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2_4x5.npz')







import spm1d




#(0) Load dataset:
dataset    = spm1d.data.uv0d.anova2rm.Antidepressant()
# dataset    = spm1d.data.uv0d.anova2rm.RSXLTraining()
# dataset    = spm1d.data.uv0d.anova2rm.SocialNetworks()
# dataset    = spm1d.data.uv0d.anova2rm.Southampton2rm()
# dataset    = spm1d.data.uv0d.anova2rm.SPM1D3x3()
# dataset    = spm1d.data.uv0d.anova2rm.SPM1D3x4()
# dataset    = spm1d.data.uv0d.anova2rm.SPM1D3x5()
# dataset    = spm1d.data.uv0d.anova2rm.SPM1D4x4()
# dataset    = spm1d.data.uv0d.anova2rm.SPM1D4x5()
y,A,B,SUBJ = dataset.get_data()
print dataset





#(1) Run ANOVA:
F = spm1d.stats.anova2rm(y, A, B, SUBJ)
# F = spm1d.stats.anova2rm(y, B, A, SUBJ)
Fvalues = [f.z for f in F]
print Fvalues











import numpy as np
from matplotlib import pyplot
import spm1d




#(0) Load data:
dataset      = spm1d.data.uv1d.anova3tworm.SPM1D_ANOVA3TWORM_2x2x2()
# dataset      = spm1d.data.uv1d.anova3tworm.SPM1D_ANOVA3TWORM_2x3x4()
Y,A,B,C,SUBJ = dataset.get_data()



#(1) Conduct ANOVA:
alpha        = 0.05
FF           = spm1d.stats.anova3tworm(Y, A, B, C, SUBJ, equal_var=True)
FFi          = [F.inference(alpha)   for F in FF]


#(2) Plot results:
pyplot.close('all')
titles       = ['Main effect A',
               'Main effect B',
               'Main effect C',
               'Interaction AB',
               'Interaction AC',
               'Interaction BC',
               'Interaction ABC',
]
for i,Fi in enumerate(FFi):
	ax = pyplot.subplot(3,3,i+1)
	Fi.plot()
	ax.text(0.1, 0.85, titles[i], transform=ax.transAxes)
pyplot.show()








# coding: utf-8

import os
import numpy as np
from .. import _base




class Besier2009muscleforces(_base.DatasetHotellings2, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Besier, T. F., Fredericson, M., Gold, G. E., Beaupré, G. S., & Delp, S. L. (2009). Knee muscle forces during walking and running in patellofemoral pain patients and pain-free controls. Journal of Biomechanics, 42(7), 898–905. http://doi.org/10.1016/j.jbiomech.2009.01.032'
		self.www      = 'https://simtk.org/home/muscleforces'
		self.note     = 'Results  ', 'Pataky, T., Robinson, M., & Vanrenterghem, J. (2015). Journal of Biomechanics. Journal of Biomechanics, 48(1), 190–192. http://doi.org/10.1016/j.jbiomech.2014.09.025'
		self.datafile = os.path.join(_base.get_datafilepath(), 'Besier2009muscleforces.npz')
		Z             = np.load(self.datafile)
		self.YA       = Z['YA']
		self.YB       = Z['YB']
		Z.close()
		self.z        = None
		self.df       = None
		self.p        = None






import os
import numpy as np
from matplotlib import pyplot
import spm1d



# load dataset:
dataset = spm1d.data.uv1d.anova1.SpeedGRFcategorical()
Y,A     = dataset.get_data()
Y0      = Y[A==1]
Y1      = Y[A==2]
Y2      = Y[A==3]



# specify an arbitrary datum and an arbitrary error trajectory:
datum    = Y1.mean(axis=0)
err      = np.linspace(0.1, 2.5, datum.size)**2


# plot:
pyplot.close('all')
pyplot.plot(datum, 'b', lw=3)
spm1d.plot.plot_errorcloud(datum, err, facecolor='r', edgecolor='r')
pyplot.show()



import spm1d




#(0) Load dataset:
dataset = spm1d.data.mv0d.hotellings2.RSXLHotellings2()
# dataset = spm1d.data.mv0d.hotellings2.HELPHomeless()
yA,yB   = dataset.get_data()
print dataset



#(1) Conduct T2 test using spm1d:
T2      = spm1d.stats.hotellings2(yA, yB)
T2i     = T2.inference(alpha=0.05)
print T2i







from math import sqrt,log
import numpy as np
import _mvbase
import _spm




##########################################
#    CANONICAL CORRELATION ANALYSIS
##########################################


# def max_effect_direction_singlenode(y, x):
# 	nObserv    = y.shape[0]
# 	nCompon    = y.shape[1]
# 	x,Y        = np.matrix(x.T).T, np.matrix(y)
# 
# 	Z          = np.matrix(np.ones(nObserv)).T
# 	Rz         = np.eye(nObserv) - Z*np.linalg.inv(Z.T*Z)*Z.T
# 	xStar      = Rz * x
# 	YStar      = Rz * Y
# 
# 	p          = x.shape[1] #nContrasts
# 	m          = nObserv - p
# 	H          = YStar.T * xStar  *  np.linalg.inv( xStar.T * xStar  )  * xStar.T * YStar / p
# 	W          = YStar.T  * (np.eye(nObserv)  -  xStar*np.linalg.inv(xStar.T*xStar)*xStar.T) * YStar  / m
# 
# 	F          = np.linalg.inv(W)*H
# 	# ff         = np.linalg.eigvals(  F  )
# 	vals,vecs  = np.linalg.eig(F)
# 	ind        = np.argmax(vals)
# 	r          = vecs[:,ind]
# 	return r



def cca_single_node(y, x):
	N          = y.shape[0]
	X,Y        = np.matrix(x.T).T, np.matrix(y)
	Z          = np.matrix(np.ones(N)).T
	Rz         = np.eye(N) - Z*np.linalg.inv(Z.T*Z)*Z.T
	XStar      = Rz * X
	YStar      = Rz * Y
	p,r        = 1.0, 1.0   #nContrasts, nNuisanceFactors
	m          = N - p - r
	H          = YStar.T * XStar  *  np.linalg.inv( XStar.T * XStar  )  * XStar.T * YStar / p
	W          = YStar.T  * (np.eye(N)  -  XStar*np.linalg.inv(XStar.T*XStar)*XStar.T) * YStar  / m
	#estimate maximum canonical correlation:
	F          = np.linalg.inv(W)*H
	ff         = np.linalg.eigvals(  F  )
	fmax       = float( np.real(ff.max()) )
	r2max      = fmax * p  / (m + fmax*p)
	rmax       = sqrt(r2max)
	### compute test statistic:
	m          = y.shape[1]
	x2         = -(N-1-0.5*(m+2)) * log(  (1-rmax**2) )
	# df         = m
	return x2



# def cca(y, x):
# 	X2         = np.array([cca_single_node(y[:,q,:], x)   for q in range(y.shape[1])])
# 	R          = _mvbase._get_residuals_regression(y, x)
# 	fwhm       = _mvbase._fwhm(R)
# 	resels     = _mvbase._resel_counts(R, fwhm)
# 	df         = 1, y.shape[2]
# 	return _spm.SPM_X2(X2, df, fwhm, resels, None, None, R)
# 	
	

def _cca_single_node_efficient(y, x, Rz, XXXiX):
	N          = y.shape[0]
	Y          = np.matrix(y)
	YStar      = Rz * Y
	p,r        = 1.0, 1.0   #nContrasts, nNuisanceFactors
	m          = N - p - r
	H          = YStar.T * XXXiX * YStar / p
	W          = YStar.T  * (np.eye(N)  -  XXXiX) * YStar  / m
	#estimate maximum canonical correlation:
	F          = np.linalg.inv(W)*H
	ff         = np.linalg.eigvals(  F  )
	fmax       = float( np.real(ff.max()) )
	r2max      = fmax * p  / (m + fmax*p)
	rmax       = sqrt(r2max)
	### compute test statistic:
	m          = y.shape[1]
	x2         = -(N-1-0.5*(m+2)) * log(  (1-rmax**2) )
	# df         = m
	return x2


def cca(Y, x):
	'''
	Canonical correlation analysis (CCA).
	
	:Parameters:
		- *Y* --- A list or tuple of (J x Q) numpy arrays
		- *x* --- (J x 1) list or array (independent variable)

	
	:Returns:
		- X2 : An **spm1d._spm.SPM_X2** instance
	
	:Note:
		-  Currently only a univariate 0D independent variable (x) is supported.
	'''
	N          = Y.shape[0]
	X          = np.matrix(x.T).T
	Z          = np.matrix(np.ones(N)).T
	Rz         = np.eye(N) - Z*np.linalg.inv(Z.T*Z)*Z.T
	XStar      = Rz * X
	XXXiX      = XStar  *  np.linalg.inv( XStar.T * XStar  )  * XStar.T
	if Y.ndim==2:
		X2         = _cca_single_node_efficient(Y, x, Rz, XXXiX)
		df         = 1, Y.shape[1]
		return _spm.SPM0D_X2(X2, df)
	else:
		X2         = np.array([_cca_single_node_efficient(Y[:,q,:], x, Rz, XXXiX)   for q in range(Y.shape[1])])
		R          = _mvbase._get_residuals_regression(Y, x)
		fwhm       = _mvbase._fwhm(R)
		resels     = _mvbase._resel_counts(R, fwhm)
		df         = 1, Y.shape[2]
		return _spm.SPM_X2(X2, df, fwhm, resels, None, None, R)







from matplotlib import pyplot
import spm1d



#(0) Load dataset:
dataset      = spm1d.data.uv1d.anova1.SpeedGRFcategorical()
dataset      = spm1d.data.uv1d.anova1.Weather()
Y,A          = dataset.get_data()



#(1) Run ANOVA:
alpha        = 0.05
F            = spm1d.stats.anova1(Y, A, equal_var=False)
Fi           = F.inference(alpha, interp=False)
print Fi
### alternative syntax:
# Y0,Y1,Y2     = [Y[A==u] for u in np.unique(A)]
# F            = spm1d.stats.anova1((Y0,Y1,Y2), equal_var=False)


#(2) Plot results:
pyplot.close('all')
Fi.plot()
Fi.plot_threshold_label(bbox=dict(facecolor='w'))
# pyplot.ylim(-1, 500)
pyplot.xlabel('Time (%)', size=20)
pyplot.title(r'Critical threshold at $\alpha$=%.2f:  $F^*$=%.3f' %(alpha, Fi.zstar))
pyplot.show()









import spm1d




#(0) Load dataset:
dataset      = spm1d.data.uv0d.anova3onerm.NYUCaffeine()
dataset      = spm1d.data.uv0d.anova3onerm.Southampton3onerm()
y,A,B,C,SUBJ = dataset.get_data()
print dataset



#(1) Run ANOVA:
F = spm1d.stats.anova3onerm(y, A, B, C, SUBJ, equal_var=True)
Fvalues = [f.z for f in F]
print Fvalues






import numpy as np
from matplotlib import pyplot
import spm1d






#first-level SPM analysis: (within-subject effects)
nSubj         = 10
BETA          = []  #regression slopes
for subj in range(nSubj):
	dataset   = spm1d.data.uv1d.regress.SpeedGRF(subj=subj)
	Y,x       = dataset.get_data()
	t         = spm1d.stats.regress(Y, x) #conduct linear regression
	BETA.append( t.beta[0] )  #retrieve the regression slope
BETA          = np.array(BETA)


#plot:
pyplot.close('all')
ax            = pyplot.axes( (0.15, 0.15, 0.8, 0.8) )
pyplot.plot(BETA.T, color='k')
ax.axhline(y=0, color='k', linewidth=1, linestyle=':')
ax.set_xlabel('Time (% stance)')
ax.set_ylabel(r'$\beta_0$   $(BW / ms^{-1})$')
pyplot.show()





import numpy as np
from .. import _base


def _here_stack(Y):
	m,n  = Y.shape
	y    = np.hstack( Y.T )
	A    = np.hstack(  [ [i]*m for i in range(n) ]  )
	SUBJ = np.hstack(  range(m)*n  )
	return y, A, SUBJ






class Abdi2010(_base.DatasetANOVA1rm):
	def _set_values(self):
		self.cite    = 'Abdi H (2010). The Greenhouse-Geisser correction. In Neil Salkind (Ed.), Encyclopedia of Research Design. Thousand Oaks, CA: Sage. 2010'
		self.www     = 'https://www.utdallas.edu/~herve/abdi-GreenhouseGeisser2010-pretty.pdf'
		
		Y  = np.array([
			[76, 64, 34, 26],
			[60, 48, 46, 30],
			[58, 34, 32, 28],
			[46, 46, 32, 28],
			[30, 18, 36, 28],
		])
		Y, A, SUBJ = _here_stack(Y)
		self.Y     = Y
		self.A     = A
		self.SUBJ  = SUBJ
		self.z     = 5.36
		self.df    = (3, 12)
		self.dfGG  = 1.34, 5.35
		self.dfGGX = 1, 4
		self.dfHF  = 1.76, 7.04
		self.p     = 0.018
		self.pGG   = 0.059
		self.pGGX  = 0.081
		self.pHF   = 0.041



class Groceries(_base.DatasetANOVA1rm):
	def _set_values(self):
		self.datafile = 'http://ww2.coastal.edu/kingw/statistics/R-tutorials/text/groceries.txt'
		self.www      = 'http://ww2.coastal.edu/kingw/statistics/R-tutorials/repeated.html'
		Y  = np.array([
			[1.17, 1.78, 1.29, 1.29],
			[1.77, 1.98, 1.99, 1.99],
			[1.49, 1.69, 1.79, 1.59],
			[0.65, 0.99, 0.69, 1.09],
			[1.58, 1.70, 1.89, 1.89],
			[3.13, 3.15, 2.99, 3.09],
			[2.09, 1.88, 2.09, 2.49],
			[0.62, 0.65, 0.65, 0.69],
			[5.89, 5.99, 5.99, 6.99],
			[4.46, 4.84, 4.99, 5.15],
		])
		Y, A, SUBJ = _here_stack(Y)
		self.Y     = Y
		self.A     = A
		self.SUBJ  = SUBJ
		self.z     = 4.344
		self.df    = (3, 27)
		self.p     = 0.0127



class Imacelebrity(_base.DatasetANOVA1rm):
	def _set_values(self):
		self.www      = 'http://discoveringstatistics.com/docs/repeatedmeasures.pdf'
		Y  = np.array([
			[8, 7, 1, 6],
			[9, 5, 2, 5],
			[6, 2, 3, 8],
			[5, 3, 1, 9],
			[8, 4, 5, 8],
			[7, 5, 6, 7],
			[10, 2, 7, 2],
			[12, 6, 8, 1],
		])
		Y, A, SUBJ = _here_stack(Y)
		self.Y     = Y
		self.A     = A
		self.SUBJ  = SUBJ
		self.z     = 3.794
		self.df    = (3, 21)
		self.dfGG  = 1.599, 11.190
		self.dfGGX = 1, 7
		self.dfHF  = 1.997, 13.981
		self.p     = 0.026
		self.pGG   = 0.059
		self.pGGX  = 0.092
		self.pHF   = 0.048



class Southampton1rm(_base.DatasetANOVA1rm):
	def _set_values(self):
		self.www   = 'http://www.southampton.ac.uk/~cpd/anovas/datasets/Doncaster&Davey%20-%20Model%206_1%20One%20factor%20repeated%20measures.txt'
		self.Y     = np.array([-4.4297, 4.7513, 3.2971, -1.4606, 4.8458, 5.1163, 6.0739, -1.9225, 6.1542, 10.4794, 12.4438, 9.2150])
		self.A     = np.array([1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3])
		self.SUBJ  = np.array([1, 2, 3, 4, 1, 2, 3, 4, 1, 2, 3, 4])
		self.z     = 12.36
		self.df    = (2, 6)
		self.p     = 0.007










# coding: utf-8

import os
import numpy as np
from .. import _base




class Dorn2012(_base.DatasetMANOVA1, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Dorn, T. W., Schache, A. G., & Pandy, M. G. (2012). Muscular strategy shift in human running: dependence of running speed on hip and ankle muscle performance. Journal of Experimental Biology, 215(11), 1944–1956. http://doi.org/10.1242/jeb.064527'
		self.www      = 'https://simtk.org/home/runningspeeds'
		self.datafile = os.path.join(_base.get_datafilepath(), 'Dorn2012.npz')
		self.note     = 'Note     ', 'Unpublished results'
		### load and parse data:
		Z             = np.load(self.datafile)
		Y,FOOT,SPEED  = Z['Y'], Z['FOOT'], Z['SPEED']
		Z.close()
		### choose one foot:
		Y,SPEED       = Y[FOOT==0], SPEED[FOOT==0]
		A,u           = np.zeros(SPEED.size), np.unique(SPEED)
		for i,uu in enumerate(u):
			A[SPEED==uu] = i
		self.Y        = Y
		self.A        = np.asarray(A, dtype=int)
		self.z        = None
		self.df       = None
		self.p        = None






'''
Restricted maximum likelihood estimates

(This and all modules whose names start with underscores
are not meant to be accessed directly by the user.)


The module contains functions which correct for non-sphericity.

The code is based on the Matlab routines in "spm_reml.m" as
released with SPM8 at www.fil.ion.ucl.ac.uk/spm/.

Specific version details:
    spm_reml.m
    $Id: spm_reml.m 1165 2008-02-22 12:45:16Z guillaume $
    authors: John Ashburner & Karl Friston
'''

# Copyright (C) 2014  Todd Pataky
# _reml.py version: 0.2.0001 (2014/05/25)


import itertools
from copy import deepcopy
from math import sqrt,exp,log
import numpy as np



eps         = np.finfo(float).eps


def _rank(A, tol=None):
	'''
	This is a slight modification of np.linalg.matrix_rank.
	The tolerance performs poorly for some matrices
	Here the tolerance is boosted by a factor of ten for improved performance.
	'''
	M = np.asarray(A)
	S = np.linalg.svd(M, compute_uv=False)
	if tol is None:
		tol = 10 * S.max() * max(M.shape) * np.finfo(M.dtype).eps
	rank = sum(S > tol)
	return rank



def estimate_df_T(Y, X, eij, Q):
	n,s         = Y.shape
	### ReML estimates:
	trRV        = n - _rank(X)
	ResSS       = (np.asarray(eij)**2).sum(axis=0)
	q           = np.diag(np.sqrt( trRV / ResSS )).T
	Ym          = Y*q
	YY          = Ym*Ym.T / s
	V,h         = reml(YY, X, Q)
	V           = V * (n / np.trace(V))
	### effective degrees of freedom:
	trRV,trRVRV = traceRV(V, X)
	df          = trRV**2 / trRVRV
	return df



# def estimate_df_anova1(Y, X, eij):
# 	n,s           = Y.shape                                #numbers ofresponses and nodes
# 	nnLevels      = np.asarray(X, dtype=int).sum(axis=0)   #number of responses in each level
# 	rankX         = _rank(X)
# 	### create contrast matrix:
# 	c             = np.matrix(np.zeros((rankX, rankX*(rankX-1) )))
# 	for col,(i0,i1) in enumerate(itertools.combinations(range(rankX),2)):
# 		c[i0,col] = 1
# 		c[i1,col] = -1
# 	c[:,col+1:]   = -c[:,:col+1]
# 	### create non-sphericity components matrix:
# 	Q,i0          = [], 0
# 	for nn in nnLevels:
# 		QQ        = np.zeros((n,n))
# 		QQ[i0:i0+nn,i0:i0+nn]  =  np.eye(nn)
# 		i0       += nn
# 		Q.append( np.matrix(QQ) )
# 	### ReML estimates:
# 	trRV          = n - rankX
# 	ResSS         = (np.asarray(eij)**2).sum(axis=0)
# 	q             = np.diag(np.sqrt( trRV / ResSS )).T
# 	Ym            = Y*q
# 	YY            = Ym*Ym.T / s
# 	V,h           = reml(YY, X, Q)
# 	V            *= (n / np.trace(V))
# 	### effective degrees of freedom (denominator):
# 	trRV,trRVRV   = traceRV(V, X)
# 	df2           = trRV**2 / trRVRV
# 	### effective degrees of freedom (numerator):
# 	trMV,trMVMV   = traceMV(V, X, c)
# 	df1           =  trMV**2 / trMVMV
# 	df1           = max(df1,1.0)
# 	return df1, df2


# def estimate_df_anova2(Y, A, B, X, eij, C):
# 	J,s       = Y.shape
# 	rankX     = _rank(X)
# 	### construct non-sphericity components:
# 	uA,uB     = np.unique(A), np.unique(B)
# 	nA,nB     = uA.size, uB.size
# 	Q         = []
# 	for u in uA:
# 		Q.append( np.diag((A==u) +0) )
# 	for u in uB:
# 		Q.append( np.diag((B==u) +0) )
# 	### ReML estimates:
# 	trRV      = J - rankX
# 	ResSS     = (np.asarray(eij)**2).sum(axis=0)
# 	q         = np.diag(np.sqrt( trRV / ResSS )).T
# 	Ym        = Y*q
# 	YY        = Ym*Ym.T / s
# 	V,h       = reml(YY, X, Q)
# 	V        *= (J / np.trace(V))
# 	### effective degrees of freedom (denominator):
# 	trRV,trRVRV = traceRV(V, X)
# 	df2         = trRV**2 / trRVRV
# 	### effective degrees of freedom (numerator):
# 	df1         = []
# 	for CC in C:
# 		trMV,trMVMV = traceMV(V, X, CC.T)
# 		df1.append( trMV**2 / trMVMV )
# 	df1       = [max(x,1.0)  for x in df1]
# 	return df1, df2



def estimate_df_anova1(Y, X, eij, Q, c):
	n,s           = Y.shape                                #numbers ofresponses and nodes
	rankX         = _rank(X)
	### ReML estimates:
	trRV          = n - rankX
	ResSS         = (np.asarray(eij)**2).sum(axis=0)
	q             = np.diag(np.sqrt( trRV / ResSS )).T
	Ym            = Y*q
	YY            = Ym*Ym.T / s
	V,h           = reml(YY, X, Q)
	V            *= (n / np.trace(V))
	### effective degrees of freedom (denominator):
	trRV,trRVRV   = traceRV(V, X)
	df2           = trRV**2 / trRVRV
	### effective degrees of freedom (numerator):
	trMV,trMVMV   = traceMV(V, X, c)
	df1           =  trMV**2 / trMVMV
	df1           = max(df1,1.0)
	return df1, df2
	# df1,df2 = 2,2


def estimate_df_anova2(Y, X, eij, Q, C):
	J,s       = Y.shape
	rankX     = _rank(X)
	### ReML estimates:
	trRV      = J - rankX
	ResSS     = (np.asarray(eij)**2).sum(axis=0)
	q         = np.diag(np.sqrt( trRV / ResSS )).T
	Ym        = Y*q
	YY        = Ym*Ym.T / s
	V,h       = reml(YY, X, Q)
	V        *= (J / np.trace(V))
	### effective degrees of freedom (denominator):
	trRV,trRVRV = traceRV(V, X)
	df2         = trRV**2 / trRVRV
	### effective degrees of freedom (numerator):
	df1         = []
	for CC in C:
		trMV,trMVMV = traceMV(V, X, CC)
		df1.append( trMV**2 / trMVMV )
	df1       = [max(x,1.0)  for x in df1]
	return df1, df2


def reml(YY, X, Q, N=1, K=128):
	n     = X.shape[0]
	W     = deepcopy(Q)
	
	q     = np.asarray(np.all(YY<np.inf, axis=0)).flatten()
	Q     = [QQ[q,:][:,q]   for QQ in Q]

	m     = len(Q)
	h     = np.matrix([float(np.any(np.diag(QQ)))  for QQ in Q]).T
	X0    = np.linalg.qr(X)[0]
	dFdhh = np.zeros((m,m))

	hE   = np.matrix(np.zeros((m,1)))
	hP   = np.eye(m)/exp(32)

	dF  = np.inf
	for k in range(K):
		C    = np.matrix(np.zeros((n,n)))
		for i in range(m):
			C   += Q[i] * float(h[i])
		iC   = np.linalg.inv(C) + np.eye(n)/exp(32)
		iCX  = iC*X0
		Cq   = np.linalg.inv(X0.T*iCX)
	
		# Gradient dF/dh (first derivatives)
		P    = iC - iCX*Cq*iCX.T
		U    = np.eye(n) - P*YY/N
	
		PQ   = [P*QQ for QQ in Q]
		dFdh = np.matrix([-np.trace(PQQ*U)*0.5*N   for PQQ in PQ]).T
		
		# Expected curvature E{dF/dhh} (second derivatives)
		for i in range(m):
			for j in range(m):
		            dFdhh[i,j] = -np.trace(PQ[i]*PQ[j])*0.5*N
		            dFdhh[j,i] =  dFdhh[i,j]

		#add hyper-priors
		e     = h - hE
		dFdh -= hP*e
		dFdhh -= hP
		
		# Fisher scoring
		dh    = -np.linalg.inv(dFdhh)*dFdh / log(k+3)
		h    += dh
		dF    = float(dFdh.T*dh)
		
		#final covariance estimate (with missing data points)
		if dF < 0.1:
			V     = 0
			for i in range(m):
				V += Q[i]*float(h[i])
			return V, h





	

	


def traceMV(V, X, c):
	rankX       = _rank(X)
	u,ds,v      = np.linalg.svd(X)
	u           = np.matrix(u[:,:rankX])
	ukX1o       = (np.matrix(np.diag(1/ds)) * np.matrix(v).T)*c
	ukX1o       = ukX1o[:rankX]
	X1o         = u * ukX1o
	###
	rnk1        = _rank(X1o)
	u1,ds1,v1   = np.linalg.svd(X1o)
	u1          = np.matrix(u1[:,:rnk1])
	Vu          = V*u1
	trMV        = (np.asarray(u1)*np.asarray(Vu)).sum()
	trMVMV      = np.linalg.norm(u1.T*Vu,  ord='fro')**2
	return trMV, trMVMV

def traceRV(V, X):
	# rk   = _rank(X);
	rk   = np.linalg.matrix_rank(X)
	sL   = X.shape[0]
	u    = np.linalg.qr(X)[0]

	Vu      = V*u
	trV     = np.trace(V)
	trRVRV  = np.linalg.norm(V,  ord='fro')**2
	trRVRV -= 2*np.linalg.norm(Vu, ord='fro')**2
	trRVRV += np.linalg.norm(u.T*Vu, ord='fro')**2
	trMV    = np.sum(np.array(u)*np.array(Vu))
	trRV    = trV - trMV
	return trRV, trRVRV



import numpy as np
from scipy import stats
import spm1d




#(0) Load dataset:
dataset = spm1d.data.uv0d.t1.RSWeightReduction()
dataset = spm1d.data.uv0d.t1.ColumbiaSalmonella()
y,mu    = dataset.get_data()
print dataset





#(1) Conduct t test using spm1d:
spmt    = spm1d.stats.ttest(y, mu)
spmti   = spmt.inference(0.05, two_tailed=False)
print spmti



#(2) Compare to scipy.stats result:
t,p     = stats.ttest_1samp(y, mu)
p       = 0.5*p  #one-tailed inference
print 'scipy.stats result:\n   t = %.5f\n   p = %.5f' %(t,p)








# coding: utf-8

import os
import numpy as np
from .. import _base




class SpeedGRFcategorical(_base.DatasetANOVA1, _base.Dataset1D):
	def __init__(self, subj=0):
		self.subj     = int(subj)
		super(SpeedGRFcategorical, self).__init__()
		
	
	def _set_values(self):
		self.cite     = 'Pataky, T. C., Caravaggi, P., Savage, R., Parker, D., Goulermas, J., Sellers, W., & Crompton, R. (2008). New insights into the plantar pressure correlates of walking speed using pedobarographic statistical parametric mapping (pSPM). Journal of Biomechanics, 41(9), 1987–1994.'
		fnameY        = os.path.join(_base.get_datafilepath(), 'ex_grf_subj%03d.npy'%self.subj)
		fnameX        = os.path.join(_base.get_datafilepath(), 'ex_grf_speeds_cond.npy')
		self.datafile = fnameY
		self.Y        = np.load(fnameY)
		self.A        = np.load(fnameX)[:,self.subj]
		self.z        = None
		self.df       = None
		self.p        = None



class Weather(_base.DatasetANOVA1, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Ramsay JO, Silverman BW (2005). Functional Data Analysis (Second Edition), Springer, New York.'
		self.www      = 'http://www.psych.mcgill.ca/misc/fda/ex-weather-a1.html'
		fname         = os.path.join(_base.get_datafilepath(), 'weather.npz')
		Z             = np.load(fname)
		y0,y1,y2,y3   = Z['y0'], Z['y1'], Z['y2'], Z['y3']
		Z.close()
		self.datafile = fname
		self.Y        = np.vstack([y0,y1,y2,y3])
		self.A        = np.array( [0]*y0.shape[0] + [1]*y1.shape[0] + [2]*y2.shape[0] + [3]*y3.shape[0] )
		self.z        = None
		self.df       = None
		self.p        = None






import spm1d




#(0) Load dataset:
dataset    = spm1d.data.uv0d.anova2onerm.Santa23()
dataset    = spm1d.data.uv0d.anova2onerm.Southampton2onerm()
# dataset    = spm1d.data.uv0d.anova2onerm.RSXLDrug()
# dataset    = spm1d.data.uv0d.anova2onerm.SPM1D3x3()
# dataset    = spm1d.data.uv0d.anova2onerm.SPM1D3x4()
# dataset    = spm1d.data.uv0d.anova2onerm.SPM1D3x4A()
# dataset    = spm1d.data.uv0d.anova2onerm.SPM1D3x5()
# dataset    = spm1d.data.uv0d.anova2onerm.SPM1D4x4()
# dataset    = spm1d.data.uv0d.anova2onerm.SPM1D4x5()
y,A,B,SUBJ = dataset.get_data()
print dataset



#(1) Run ANOVA:
F = spm1d.stats.anova2onerm(y, A, B, SUBJ)
Fvalues = [f.z for f in F]
DF = [f.df for f in F]
print Fvalues
print DF







import spm1d




#(0) Load dataset:
dataset    = spm1d.data.uv0d.anova2nested.QIMacros()
dataset    = spm1d.data.uv0d.anova2nested.SouthamptonNested1()
y,A,B      = dataset.get_data()
print dataset


#(1) Run ANOVA:
F = spm1d.stats.anova2nested(y, A, B)
Fvalues = [f.z for f in F]
DF = [f.df for f in F]
print Fvalues
print DF





# __version__ = '0.0.0005'  #2014.08.12


from math import sqrt,log
import numpy as np
import _mvbase
import _spm


eps        = np.finfo(float).eps   #smallest float, used to avoid divide-by-zero errors



def manova1_single_node(Y, GROUP):
	### assemble counts:
	u           = np.unique(GROUP)
	nGroups     = u.size
	nResponses  = Y.shape[0]
	nComponents = Y.shape[1]
	### create design matrix:
	X           = np.zeros((nResponses, nGroups))
	ind0        = 0
	for i,uu in enumerate(u):
		n       = (GROUP==uu).sum()
		X[ind0:ind0+n, i] = 1
		ind0   += n
	### SS for original design:
	Y,X   = np.matrix(Y), np.matrix(X)
	b     = np.linalg.pinv(X)*Y
	R     = Y - X*b
	R     = R.T*R
	### SS for reduced design:
	X0    = np.matrix(  np.ones(Y.shape[0])  ).T
	b0    = np.linalg.pinv(X0)*Y
	R0    = Y - X0*b0
	R0    = R0.T*R0
	### Wilk's lambda:
	lam   = np.linalg.det(R) / (np.linalg.det(R0) + eps)
	### test statistic:
	N,p,k = float(nResponses), float(nComponents), float(nGroups)
	x2    = -((N-1) - 0.5*(p+k)) * log(lam)
	df    = p*(k-1)
	# return lam, x2, df
	return x2



# def manova1(Y, GROUP):
# 	nNodes  = Y.shape[1]
# 	X2      = np.array([manova1_single_node(Y[:,i,:], GROUP)  for i in range(nNodes)])
# 	R       = _mvbase._get_residuals_manova1(Y, GROUP)
# 	fwhm    = _mvbase._fwhm(R)
# 	resels  = _mvbase._resel_counts(R, fwhm)
# 	df      = Y.shape[2]*( np.unique(GROUP).size -1 )
# 	return _spm.SPM_X2(X2, (1,df), fwhm, resels, None, None, R)
# 



def _manova1_single_node_efficient(Y, GROUP, X, Xi, X0, X0i, nGroups):
	### SS for original design:
	Y     = np.matrix(Y)
	b     = Xi*Y
	R     = Y - X*b
	R     = R.T*R
	### SS for reduced design:
	b0    = X0i*Y
	R0    = Y - X0*b0
	R0    = R0.T*R0
	### Wilk's lambda:
	lam   = np.linalg.det(R) / (np.linalg.det(R0) + eps)
	# print lam
	### test statistic:
	(N,p),k = Y.shape, float(nGroups)
	x2    = -((N-1) - 0.5*(p+k)) * log(lam)
	# df    = p*(k-1)
	# return lam, x2, df
	return x2



def manova1(Y, A, equal_var=True):
	'''
	Two-way repeated-measures ANOVA.
	
	:Parameters:
		- *Y* --- (J x Q x I) numpy array
		- *A* --- (J x 1) vector of integer labels for Factor A
		- *equal_var* --- If *True*, equal group variance will be assumed

	:Returns:
		- X2 : An **spm1d._spm.SPM_X2** instance

	:Note:
		- Non-sphericity correction not implemented. Equal variance must be assumed by setting "equal_var=True".
	'''
	if equal_var is not True:
		raise( UserWarning('Non-sphericity correction not implemented. To continue you must assume equal variance and set "equal_var=True".') )
	u           = np.unique(A)
	nGroups     = u.size
	nResponses  = Y.shape[0]
	### create design matrix:
	X           = np.zeros((nResponses, nGroups))
	ind0        = 0
	for i,uu in enumerate(u):
		X[A==uu, i] = 1
	X           = np.matrix(X)
	Xi          = np.linalg.pinv(X)
	### reduced design:
	X0          = np.matrix(  np.ones(Y.shape[0])  ).T
	X0i         = np.linalg.pinv(X0)
	### compute test statistic:
	if Y.ndim==2:
		nComponents = Y.shape[1]
		X2          = _manova1_single_node_efficient(Y, A, X, Xi, X0, X0i, nGroups)
		df          = nComponents*( nGroups-1 )
		return _spm.SPM0D_X2(X2, (1,df))
	else:
		nNodes      = Y.shape[1]
		nComponents = Y.shape[2]
		X2          = np.array([_manova1_single_node_efficient(Y[:,i,:], A, X, Xi, X0, X0i, nGroups)  for i in range(nNodes)])
		R           = _mvbase._get_residuals_manova1(Y, A)
		fwhm        = _mvbase._fwhm(R)
		resels      = _mvbase._resel_counts(R, fwhm)
		df          = nComponents*( nGroups-1 )
		return _spm.SPM_X2(X2, (1,df), fwhm, resels, None, None, R)




	






'''
Statistics module.

This module contains functions for conducting classical hypothesis testing on a set of 1D continua.

For all tests the dependent variable **Y** must be a NumPy array, with dimensions::

* J :  number of observations
* Q :  number of field nodes
* I :  number of vector components

Specifically:

* Univariate 0D tests:  **Y** should be ( J x 1 )
* Multivariate 0D tests:  **Y** should be ( J x I )
* Univariate 1D tests:  **Y** should be ( J x Q )
* Multivariate 1D tests:  **Y** should be ( J x Q x I )

'''

import _spm

from t import ttest, ttest_paired, ttest2, regress, glm

from anova import anova1,anova1rm
from anova import anova2,anova2nested,anova2rm,anova2onerm
from anova import anova3,anova3nested,anova3rm,anova3tworm,anova3onerm

from hotellings import hotellings, hotellings_paired, hotellings2
from cca import cca
from manova import manova1




import numpy as np
from matplotlib import pyplot
import spm1d




#(0) Load data:
dataset      = spm1d.data.uv1d.anova3nested.SPM1D_ANOVA3NESTED_2x2x2()
# dataset      = spm1d.data.uv1d.anova3nested.SPM1D_ANOVA3NESTED_2x4x2()
Y,A,B,C      = dataset.get_data()



#(1) Conduct ANOVA:
alpha        = 0.05
FF           = spm1d.stats.anova3nested(Y, A, B, C, equal_var=True)
FFi          = [F.inference(alpha)   for F in FF]


#(2) Plot results:
pyplot.close('all')
titles       = ['Main effect A',
               'Main effect B',
               'Main effect C',
]
for i,Fi in enumerate(FFi):
	ax = pyplot.subplot(2,2,i+1)
	Fi.plot()
	ax.text(0.1, 0.85, titles[i], transform=ax.transAxes)
pyplot.show()









import os
import numpy as np
from .. import _base




class _SPM1D_ANOVA3TWORM_DATASET(_base.DatasetANOVA3rm, _base.Dataset1D):
	def _set_values(self):
		self._set_datafile()
		Z             = np.load(self.datafile)
		self.Y,self.A,self.B,self.C,self.SUBJ = Z['Y'], Z['A'], Z['B'], Z['C'], Z['SUBJ']
		Z.close()


class SPM1D_ANOVA3TWORM_2x2x2(_SPM1D_ANOVA3TWORM_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova3tworm_2x2x2.npz')
class SPM1D_ANOVA3TWORM_2x3x4(_SPM1D_ANOVA3TWORM_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova3tworm_2x3x4.npz')





'''
SPM module

(This and all modules whose names start with underscores
are not meant to be accessed directly by the user.)

This module contains class definitions for raw SPMs (raw test statistic continua)
and inference SPMs (thresholded test statistic).
'''

# Copyright (C) 2014  Todd Pataky
# _spm.py version: 0.2.0006 (2014/07/09)


import numpy as np
from scipy import ndimage,optimize,stats
from .. plot import plot_spm, plot_spm_design
from .. plot import plot_spmi, plot_spmi_p_values, plot_spmi_threshold_label
import rft1d



def df2str(df):
	return str(df) if not df%1 else '%.3f'%df
def dflist2str(dfs):
	return '(%s, %s)' %(df2str(dfs[0]), df2str(dfs[1]))
def p2string(p):
	return '<0.001' if p<0.0005 else '%.03f'%p
def plist2string(pList):
	s      = ''
	if len(pList)>0:
		for p in pList:
			s += p2string(p)
			s += ', '
		s  = s[:-2]
	return s
def _set_docstr(childfn, parentfn, args2remove=None):
	docstr      =  parentfn.__doc__
	if args2remove!=None:
		docstrlist0 = docstr.split('\n\t')
		docstrlist1 = []
		for s in docstrlist0:
			if not np.any([s.startswith('- *%s*'%argname) for argname in args2remove]):
				docstrlist1.append(s)
		docstrlist1 = [s + '\n\t'  for s in docstrlist1]
		docstr  = ''.join(docstrlist1)
	childfn.__func__.__doc__ = docstr


eps    = np.finfo(float).eps   #smallest float, used to avoid divide-by-zero errors






'''
#################
(0)  CLUSTER CLASS DEFINITION
#################
'''
class Cluster(object):
	def __init__(self, m, mr, h, xy, p):
		self.extent  = m       #cluster size (absolute)
		self.extentR = mr      #cluster size (resels)
		self.h       = h       #cluster height (minimum above threshold)
		self.xy      = tuple(xy)      #cluster centroid
		self.P       = p       #probability value (based on h and extentR)
	def __repr__(self):
		s        = ''
		s       += 'Cluster at location: (%.3f, %.3f)\n' %self.xy
		s       += '   extent          :  %d\n' %self.extent
		s       += '   extent (resels) :  %.5f\n' %self.extentR
		s       += '   height (min)    :  %.5f\n' %self.h
		if self.P==None:
			s   += '   P               :  None\n\n'
		else:
			s   += '   P               :  %.5f\n\n' %self.P
		return s





'''
#################
(1)  SPM (0D) CLASS DEFINITIONS
#################
'''



class _SPM0D(object):
	def __init__(self, STAT, z, df):
		self.STAT           = STAT             #test statistic ("T" or "F")
		self.z              = float(z)         #test statistic
		self.df             = df               #degrees of freedom
		self.isanova        = False
		self.isregress      = False

	def __repr__(self):
		stat     = 't' if self.STAT=='T' else self.STAT
		s        = ''
		s       += 'SPM{%s} (0D)\n' %stat
		if self.isanova:
			s   += '   SPM.SS       : (%s,%s)\n' %self.ss
			s   += '   SPM.df       : (%s,%s)\n' %self.df
			s   += '   SPM.MS       : (%s,%s)\n' %self.ms
			s   += '   SPM.z        :  %.5f\n' %self.z
		else:
			s   += '   SPM.z      :  %.5f\n' %self.z
			s   += '   SPM.df     :  %s\n' %dflist2str(self.df)
		if self.isregress:
			s   += '   SPM.r      :  %.5f\n'   %self.r
		s       += '\n'
		return s


class _SPM0Dinference(_SPM0D):
	def __init__(self, spm, alpha, zstar, p, two_tailed=False):
		_SPM0D.__init__(self, spm.STAT, spm.z, spm.df)
		self.alpha       = alpha       #Type I error rate
		self.zstar       = zstar       #critical threshold
		self.h0reject    = abs(spm.z) > zstar if two_tailed else spm.z > zstar
		self.p           = p           #p value
		self.two_tailed  = two_tailed  #two-tailed test boolean
		self.isanova     = spm.isanova
		self.isregress   = spm.isregress
		if self.isanova:
			self.ss      = spm.ss
			self.ms      = spm.ms
		if self.isregress:
			self.r       = spm.r

	def __repr__(self):
		s        = ''
		s       += 'SPM{%s} (0D) inference\n'    %self.STAT
		s       += '   SPM.z        :  %.5f\n'   %self.z
		s       += '   SPM.df       :  %s\n'     %dflist2str(self.df)
		if self.isregress:
			s   += '   SPM.r        :  %.5f\n'   %self.r
		s       += 'Inference:\n'
		s       += '   SPM.alpha    :  %.3f\n'   %self.alpha
		s       += '   SPM.zstar    :  %.5f\n'   %self.zstar
		s       += '   SPM.h0reject :  %s\n'     %self.h0reject
		s       += '   SPM.p        :  %.5f\n'   %self.p
		return s
	


class SPM0D_F(_SPM0D):
	def __init__(self, z, df, ss=(0,0), ms=(0,0), eij=0, X0=None):
		_SPM0D.__init__(self, 'F', z, df)
		self.ss        = tuple( map(float, ss) )
		self.ms        = tuple( map(float, ms) )
		self.eij       = eij
		self.X0        = X0
		self.isanova   = True
	def inference(self, alpha=0.05):
		zstar  = stats.f.isf(alpha, self.df[0], self.df[1])
		p      = stats.f.sf(self.z, self.df[0], self.df[1])
		return SPM0Di_F(self, alpha, zstar, p)




class SPM0D_T(_SPM0D):
	def __init__(self, z, df):
		_SPM0D.__init__(self, 'T', z, df)
	def inference(self, alpha=0.05, two_tailed=True):
		a      = 0.5*alpha if two_tailed else alpha
		zstar  = stats.t.isf(a, self.df[1])
		p      = stats.t.sf( abs(self.z), self.df[1])
		p      = min(1, 2*p) if two_tailed else p
		return SPM0Di_T(self, alpha, zstar, p, two_tailed)

class SPM0D_T2(_SPM0D):
	def __init__(self, z, df):
		_SPM0D.__init__(self, 'T2', z, df)
	def inference(self, alpha=0.05):
		zstar  = rft1d.T2.isf0d(alpha, self.df)
		p      = rft1d.T2.sf0d( self.z, self.df)
		return SPM0Di_T2(self, alpha, zstar, p)

class SPM0D_X2(_SPM0D):
	def __init__(self, z, df):
		_SPM0D.__init__(self, 'X2', z, df)
	def inference(self, alpha=0.05):
		zstar  = rft1d.chi2.isf0d(alpha, self.df[1])
		p      = rft1d.chi2.sf0d( self.z, self.df[1])
		return SPM0Di_X2(self, alpha, zstar, p)



class SPM0Di_F(_SPM0Dinference):
	'''An SPM{F} (0D) inference object.'''
	pass
class SPM0Di_T(_SPM0Dinference):
	'''An SPM{T} (0D) inference object.'''
	pass
class SPM0Di_T2(_SPM0Dinference):
	'''An SPM{T2} (0D) inference object.'''
	pass
class SPM0Di_X2(_SPM0Dinference):
	'''An SPM{X2} (0D) inference object.'''
	pass






'''
#################
(3)  SPM CLASS DEFINITIONS
#################
'''


class _SPM(object):
	'''Parent class for all SPM.'''
	def __init__(self, STAT, z, df, fwhm, resels, X, beta, residuals):
		z[np.isnan(z)]      = 0
		self.STAT           = STAT             #test statistic ("T" or "F")
		self.Q              = z.size           #number of nodes (field size = Q-1)
		self.X              = X                #design matrix
		self.beta           = beta             #fitted parameters
		self.residuals      = residuals        #model residuals
		self.z              = z                #test statistic
		self.df             = df               #degrees of freedom
		self.fwhm           = fwhm             #smoothness
		self.resels         = resels           #resel counts


	def __repr__(self):
		stat     = self.STAT
		if stat == 'T':
			stat = 't'
		s        = ''
		s       += 'SPM{%s}\n' %stat
		s       += '   SPM.z      :  (1x%d) test stat field\n' %self.Q
		s       += '   SPM.df     :  %s\n' %dflist2str(self.df)
		s       += '   SPM.fwhm   :  %.5f\n' %self.fwhm
		s       += '   SPM.resels :  (%d, %.5f)\n\n\n' %tuple(self.resels)
		return s


	def inference(self, alpha=0.05, cluster_size=0, two_tailed=False, interp=True, circular=False, withBonf=True):
		a         = 0.5*alpha if two_tailed else alpha
		if self.STAT == 'T':
			zstar = rft1d.t.isf(a, self.df[1], self.Q, self.fwhm, withBonf=withBonf)
		elif self.STAT == 'F':
			zstar = rft1d.f.isf(a, self.df, self.Q, self.fwhm, withBonf=withBonf)
		elif self.STAT == 'T2':
			zstar = rft1d.T2.isf(a, self.df, self.Q, self.fwhm, withBonf=withBonf)
		elif self.STAT == 'X2':
			zstar = rft1d.chi2.isf(a, self.df[1], self.Q, self.fwhm, withBonf=withBonf)
		### compute suprathreshold cluster characteristics:
		ccalc     = rft1d.geom.ClusterMetricCalculatorInitialized(self.z, zstar, interp=interp, wrap=circular)
		extents,minima,centroids,L = ccalc.get_all()
		signs     = [1]*ccalc.n
		### compute negative cluster characteristics:
		if two_tailed:
			ccalc      = rft1d.geom.ClusterMetricCalculatorInitialized(-self.z, zstar, interp=interp, wrap=circular)
			if ccalc.n > 0:
				extents1,minima1,centroids1,L1 = ccalc.get_all()
				extents   += extents1
				minima    += (-1*np.array(minima1)).tolist()
				centroids += (np.array(centroids1)*[1,-1]).tolist()
				L         += L1
				signs     += [-1]*ccalc.n
		### set-level inference:
		nUpcrossings  = len(extents)
		p_set         = 1.0
		if nUpcrossings>0:
			minextent     = min(extents)/self.fwhm
			if self.STAT == 'T':
				p_set = rft1d.t.p_set(nUpcrossings, minextent, zstar, self.df[1], self.Q, self.fwhm, withBonf=withBonf)
			elif self.STAT == 'F':
				p_set = rft1d.f.p_set(nUpcrossings, minextent, zstar, self.df, self.Q, self.fwhm, withBonf=withBonf)
			elif self.STAT == 'T2':
				p_set = rft1d.T2.p_set(nUpcrossings, minextent, zstar, self.df, self.Q, self.fwhm, withBonf=withBonf)
			elif self.STAT == 'X2':
				p_set = rft1d.chi2.p_set(nUpcrossings, minextent, zstar, self.df[1], self.Q, self.fwhm, withBonf=withBonf)
		### cluster-level inference:
		clusters  = []
		for extent,minimum,centroid,sign in zip(extents, minima, centroids, signs):
			xR    = extent / self.fwhm
			if self.STAT == 'T':
				p = rft1d.t.p_cluster(xR, sign*minimum, self.df[1], self.Q, self.fwhm, withBonf=withBonf)
			elif self.STAT == 'F':
				p = rft1d.f.p_cluster(xR, minimum, self.df, self.Q, self.fwhm, withBonf=withBonf)
			elif self.STAT == 'T2':
				p = rft1d.T2.p_cluster(xR, minimum, self.df, self.Q, self.fwhm, withBonf=withBonf)
			elif self.STAT == 'X2':
				p = rft1d.chi2.p_cluster(xR, minimum, self.df[1], self.Q, self.fwhm, withBonf=withBonf)
			c     = Cluster(extent, xR, minimum, centroid, p)
			clusters.append(c)
		
		### assemble p values:
		nClusters = len(clusters)
		p         = [c.P for c in clusters]
		if self.STAT == 'T':
			return SPMi_T(self, alpha, zstar, nClusters, clusters, L, p_set, p, two_tailed)
		elif self.STAT == 'F':
			return SPMi_F(self, alpha, zstar, nClusters, clusters, L, p_set, p, two_tailed)
		elif self.STAT == 'T2':
			return SPMi_T2(self, alpha, zstar, nClusters, clusters, L, p_set, p, two_tailed)
		elif self.STAT == 'X2':
			return SPMi_X2(self, alpha, zstar, nClusters, clusters, L, p_set, p, two_tailed)
	
	def plot(self, **kwdargs):
		return plot_spm(self, **kwdargs)
		
	def plot_design(self, **kwdargs):
		plot_spm_design(self, **kwdargs)
		
	def toarray(self):
		return self.z.copy()










class SPM_F(_SPM):
	'''
	Create an SPM{F} continuum.
	SPM objects are instantiated in the **spm1d.stats** module.
	
	:Parameters:
	
	y : the SPM{F} continuum
	
	fwhm: estimated field smoothness (full-width at half-maximium)
	
	df : 2-tuple specifying the degrees of freedom (interest,error)
	
	resels : 2-tuple specifying the resel counts
	
	X : experimental design matrix
	
	beta : array of fitted model parameters
	
	residuals : array of residuals (used for smoothness estimation)
	
	:Returns:
	
	A **spm1d._spm.SPM_F** instance.
	
	:Methods:
	'''
	def __init__(self, z, df, fwhm, resels, X, beta, residuals, X0=None):
		_SPM.__init__(self, 'F', z, df, fwhm, resels, X, beta, residuals)
		self.X0 = X0
		
	def inference(self, alpha=0.05, cluster_size=0, interp=True, circular=False):
		'''
		Conduct statistical inference using random field theory.
		
		:Parameters:
		
		alpha        : Type I error rate (default: 0.05)
		
		cluster_size : Minimum cluster size of interest (default: 0), smaller clusters will be ignored
		:Returns:
		
		A **spm1d._spm.SPMi_F** instance.
		'''
		return _SPM.inference(self, alpha, cluster_size, two_tailed=False, interp=interp, circular=circular)



class SPM_T(_SPM):
	'''
	Create an SPM{T} continuum.
	SPM objects are instantiated in the **spm1d.stats** module.
	
	:Parameters:
	
	y : the SPM{T} continuum
	
	fwhm: estimated field smoothness (full-width at half-maximium)
	
	df : a 2-tuple specifying the degrees of freedom (interest,error)
	
	resels : a 2-tuple specifying the resel counts
	
	X : experimental design matrix
	
	beta : array of fitted model parameters
	
	residuals : array of residuals (used for smoothness estimation)
	
	:Returns:
	
	A **spm1d._spm.SPM_t** instance.
	
	:Methods:
	'''
	def __init__(self, z, df, fwhm, resels, X, beta, residuals):
		_SPM.__init__(self, 'T', z, df, fwhm, resels, X, beta, residuals)
		
	def inference(self, alpha=0.05, cluster_size=0, two_tailed=True, interp=True, circular=False):
		'''
		Conduct statistical inference using random field theory.
		
		:Parameters:
		
		alpha        : Type I error rate (default: 0.05)
		
		cluster_size : Minimum cluster size of interest (default: 0), smaller clusters will be ignored
		
		two_tailed   : Conduct two-tailed inference (default: False)
		
		:Returns:
		
		A **spm1d._spm.SPMi_T** instance.
		'''
		return _SPM.inference(self, alpha, cluster_size, two_tailed, interp, circular)




class SPM_T2(_SPM):
	def __init__(self, z, df, fwhm, resels, X, beta, residuals):
		super(SPM_T2, self).__init__('T2', z, df, fwhm, resels, X, beta, residuals)
		
	
	# def __repr__(self):
	# 	stat     = 'T2'
	# 	s        = ''
	# 	s       += 'SPM{%s}\n' %stat
	# 	s       += '   SPM.z      :  (1x%d) test stat field\n' %self.Q
	# 	s       += '   SPM.df     :  %s\n' %dflist2str(self.df)
	# 	s       += '   SPM.fwhm   :  %.5f\n' %self.fwhm
	# 	s       += '   SPM.resels :  (%d, %.5f)\n' %tuple(self.resels)
	# 	return s
	


class SPM_X2(_SPM):
	def __init__(self, z, df, fwhm, resels, X, beta, residuals):
		super(SPM_X2, self).__init__('X2', z, df, fwhm, resels, X, beta, residuals)








'''
#################
(2)  SPM INFERENCE CLASS DEFINITIONS
#################
'''


class _SPMinference(_SPM):
	'''Parent class for SPM inference objects.'''
	def __init__(self, spm, alpha, zstar, nClusters, clusters, L, p_set, p, two_tailed=False):
		_SPM.__init__(self, spm.STAT, spm.z, spm.df, spm.fwhm, spm.resels, spm.X, spm.beta, spm.residuals)
		self.alpha       = alpha       #Type I error rate
		self.zstar       = zstar       #critical threshold
		self.h0reject    = nClusters > 0
		self.nClusters   = nClusters   #number of supra-threshold clusters
		self.clusters    = clusters    #supra-threshold cluster information
		self.L           = L           #cluster labels
		self.p_set       = p_set       #set-level p value
		self.p           = p           #P values for each cluster
		self.two_tailed  = two_tailed  #two-tailed test boolean

	def __repr__(self):
		s        = ''
		s       += 'SPM{%s} inference field\n' %self.STAT
		s       += '   SPM.z         :  (1x%d) raw test stat field\n' %self.Q
		s       += '   SPM.df        :  %s\n' %dflist2str(self.df)
		s       += '   SPM.fwhm      :  %.5f\n' %self.fwhm
		s       += '   SPM.resels    :  (%d, %.5f)\n' %tuple(self.resels)
		s       += 'Inference:\n'
		s       += '   SPM.alpha     :  %.3f\n' %self.alpha
		s       += '   SPM.zstar     :  %.5f\n' %self.zstar
		s       += '   SPM.h0reject  :  %s\n' %self.h0reject
		s       += '   SPM.p_set     :  %s\n' %p2string(self.p_set)
		s       += '   SPM.p_cluster :  (%s)\n\n\n' %plist2string(self.p)
		return s
	
	def plot(self, **kwdargs):
		return plot_spmi(self, **kwdargs)

	def plot_p_values(self, **kwdargs):
		plot_spmi_p_values(self, **kwdargs)
	
	def plot_threshold_label(self, **kwdargs):
		return plot_spmi_threshold_label(self, **kwdargs)
	
	





class SPMi_T(_SPMinference):
	'''An SPM{T} inference continuum.'''
	pass
class SPMi_F(_SPMinference):
	'''An SPM{F} inference continuum.'''
	pass
class SPMi_T2(_SPMinference):
	'''An SPM{T2} inference continuum.'''
	pass
class SPMi_X2(_SPMinference):
	'''An SPM{X2} inference continuum.'''
	pass




#set docstrings:
_set_docstr(_SPM.plot, plot_spm, args2remove=['spm'])
_set_docstr(_SPMinference.plot, plot_spmi, args2remove=['spmi'])
_set_docstr(_SPMinference.plot_p_values, plot_spmi_p_values, args2remove=['spmi'])
_set_docstr(_SPMinference.plot_threshold_label, plot_spmi_threshold_label, args2remove=['spmi'])








from matplotlib import pyplot
import spm1d



#(0) Load data:
dataset      = spm1d.data.mv1d.hotellings2.Besier2009muscleforces()
YA,YB        = dataset.get_data()  #A:slow, B:fast
print dataset



#(1) Conduct test:
alpha        = 0.05
T2           = spm1d.stats.hotellings2(YA, YB)
T2i          = T2.inference(0.05)



#(2) Plot:
pyplot.close('all')
T2i.plot()
pyplot.show()






import spm1d




#(0) Load dataset:
dataset = spm1d.data.mv0d.manova1.AnimalDepression()
# dataset = spm1d.data.mv0d.manova1.Stevens2002()
y,A     = dataset.Y, dataset.A
print dataset


#(1) Conduct T2 test using spm1d:
X2      = spm1d.stats.manova1(y, A)
X2i     = X2.inference(alpha=0.05)
print X2i





import spm1d





#(0) Load dataset:
dataset   = spm1d.data.uv0d.anova3.RSItalian()
dataset   = spm1d.data.uv0d.anova3.SouthamptonFullyCrossedMixed()
y,A,B,C   = dataset.get_data()
print dataset



#(1) Run ANOVA:
F = spm1d.stats.anova3(y, A, B, C, equal_var=True)
Fvalues = [f.z for f in F]
print Fvalues





import numpy as np
from matplotlib import pyplot
import spm1d




#(0) Load data:
dataset      = spm1d.data.uv1d.anova2nested.SPM1D_ANOVA2NESTED_2x2()
# dataset      = spm1d.data.uv1d.anova2nested.SPM1D_ANOVA2NESTED_2x3()
# dataset      = spm1d.data.uv1d.anova2nested.SPM1D_ANOVA2NESTED_3x3()
# dataset      = spm1d.data.uv1d.anova2nested.SPM1D_ANOVA2NESTED_3x4()
# dataset      = spm1d.data.uv1d.anova2nested.SPM1D_ANOVA2NESTED_3x5()
# dataset      = spm1d.data.uv1d.anova2nested.SPM1D_ANOVA2NESTED_4x4()
# dataset      = spm1d.data.uv1d.anova2nested.SPM1D_ANOVA2NESTED_4x5()
Y,A,B        = dataset.get_data()



#(1) Conduct ANOVA:
alpha        = 0.05
FF           = spm1d.stats.anova2nested(Y, A, B, equal_var=True)
FFi          = [F.inference(alpha)   for F in FF]


#(2) Plot results:
pyplot.close('all')
pyplot.subplot(221);  FFi[0].plot();  pyplot.title('Main effect A')
pyplot.subplot(222);  FFi[1].plot();  pyplot.title('Main effect B')
pyplot.show()









import spm1d






#(0) Load dataset:
dataset   = spm1d.data.uv0d.anova2.Mouse()       #2x2
# dataset   = spm1d.data.uv0d.anova2.Detergent()     #2x3
# dataset   = spm1d.data.uv0d.anova2.Satisfaction()  #2x3
# dataset   = spm1d.data.uv0d.anova2.SouthamptonCrossed1()  #2x3
# dataset   = spm1d.data.uv0d.anova2.SPM1D3x3()
# dataset   = spm1d.data.uv0d.anova2.SPM1D3x4()
# dataset   = spm1d.data.uv0d.anova2.SPM1D3x5()
# dataset   = spm1d.data.uv0d.anova2.SPM1D4x4()
# dataset   = spm1d.data.uv0d.anova2.SPM1D4x5()
y,A,B     = dataset.get_data()
print dataset



#(1) Run ANOVA:
F = spm1d.stats.anova2(y, A, B, equal_var=True)
Fvalues = [f.z for f in F]
print Fvalues










import spm1d







#(0) Load dataset:
dataset = spm1d.data.uv0d.anova1.Cars()
# dataset = spm1d.data.uv0d.anova1.Sound()
# dataset = spm1d.data.uv0d.anova1.Southampton1()
# dataset = spm1d.data.uv0d.anova1.ConstructionUnequalSampleSizes()
# dataset = spm1d.data.uv0d.anova1.RSUnequalSampleSizes()
y,A     = dataset.get_data()
print dataset



#(1) Run ANOVA
F = spm1d.stats.anova1(y, A, equal_var=True)
print F







import numpy as np
from matplotlib import pyplot
import spm1d






#(0) Load data:
dataset      = spm1d.data.uv1d.anova1.SpeedGRFcategorical()
Y,A          = dataset.get_data()
Y1,Y2,Y3     = [Y[A==u] for u in np.unique(A)]


#(1) Conduct main test:
F            = spm1d.stats.anova1(Y, A, equal_var=True)
Fi           = F.inference(0.05)


#(2) Conduct Post hoc tests:
alpha        = 0.05
nTests       = 3
p_critical   = spm1d.util.p_critical_bonf(alpha, nTests)
### t statistics:
t12   = spm1d.stats.ttest2(Y1, Y2, equal_var=False)
t13   = spm1d.stats.ttest2(Y1, Y3, equal_var=False)
t23   = spm1d.stats.ttest2(Y2, Y3, equal_var=False)
### inference:
t12i  = t12.inference(alpha=p_critical, two_tailed=True)
t13i  = t13.inference(alpha=p_critical, two_tailed=True)
t23i  = t23.inference(alpha=p_critical, two_tailed=True)



#(2) Plot results:
pyplot.close('all')
pyplot.subplot(221)
Fi.plot()
Fi.plot_threshold_label(bbox=dict(facecolor='w'))
pyplot.ylim(-1, 500)
pyplot.title('Main test')

pyplot.subplot(222);  t12i.plot();  pyplot.title('Posthoc:  1 vs. 2');  pyplot.ylim(-40, 40)
pyplot.subplot(223);  t23i.plot();  pyplot.title('Posthoc:  2 vs. 3');  pyplot.ylim(-40, 40)
pyplot.subplot(224);  t13i.plot();  pyplot.title('Posthoc:  1 vs. 3');  pyplot.ylim(-40, 40)


pyplot.show()









from matplotlib import pyplot
import spm1d




#(0) Load data:
dataset  = spm1d.data.uv1d.anova1.Weather()
Y,A      = dataset.get_data()
Y0,Y1    = Y[A==0], Y[A==2]  #Atlantic and Contintental


#(1) Run stats
t = spm1d.stats.ttest2(Y0, Y1)
ti = t.inference(0.05, circular=True)
print ti


#(2) Plot:
pyplot.close('all')
ti.plot()
ti.plot_p_values()
pyplot.show()








import numpy as np







class Factor(object):
	def __init__(self, A):
		self.A            = np.asarray(A, dtype=int)        #integer vector of factor levels
		self.J            = None     #number of observations
		self.u            = None     #unique levels
		self.n            = None     #number of levels
		self.isnested     = False    #nested flag
		self.nested       = None
		self.balanced     = True
		self._00_parse()
		self._01_check_unbalanced()

	def _00_parse(self):
		self.J            = self.A.size
		self.u            = np.unique(self.A)
		self.n            = self.u.size
		
	def _01_check_unbalanced(self):
		nn  = [(self.A==uu).sum()   for uu in self.u]
		self.balanced = np.all( nn==nn[0] )

	def check_balanced(self, other):
		A,B = self.A, other.A
		N   = []
		for uA in self.u:
			for uB in other.u:
				N.append( ((A==uA) & (B==uB)).sum()  )
		return np.all( N==N[0] )


	def check_balanced_nested(self, other):
		A,B   = self.A, other.A
		N     = []
		for uA,UB in zip(self.u,other.u):
			for uB in UB:
				N.append( ((A==uA) & (B==uB)).sum()  )
		return np.all( N==N[0] )


	def check_balanced_nested3(self, other, another):
		A,B,C = self.A, other.A, another.A
		N     = []
		for uA,UB,UUC in zip(self.u,other.u,another.u):
			for uB,UC in zip(UB, UUC):
				for uC in UC:
					N.append( ((A==uA) & (B==uB) & (C==uC) ).sum()  )
		return np.all( N==N[0] )


	def check_balanced_rm(self, other):
		S,A = self.A, other.A
		N   = []
		for uS in self.u:
			UA  = np.unique(A[S==uS])
			for uA in UA:
				N.append( ((S==uS) & (A==uA)).sum()  )
		return np.all( N==N[0] )



	def get_Q(self):  #non-sphericity components
		return [np.matrix(np.diag(self.A==u), dtype=float) for u in self.u]
	
	
	def get_design_interaction(self, other):
		XAB        = []
		A,B        = self.A, other.A
		for uB in other.u[1:]:
			for uA in self.u[1:]:
				x      = np.zeros(self.J)
				x[(A==uA)&(B==uB)] =  1
				XAB.append(x)
		return np.matrix(XAB).T


	def get_design_interaction_3way(self, other, another):
		XABC       = []
		A,B,C      = self.A, other.A, another.A
		for uC in another.u[1:]:
			for uB in other.u[1:]:
				for uA in self.u[1:]:
					x      = np.zeros(self.J)
					### part one:
					x[(A==uA)&(B==uB)&(C==uC)] =  1
					XABC.append(x)
		return np.matrix(XABC).T


	def get_design_interaction_4way(self, other, another, yetanother):
		X        = []
		S,A,B,C  = self.A, other.A, another.A, yetanother.A
		for uC in yetanother.u[1:]:
			for uB in another.u[1:]:
				for uA in other.u[1:]:
					for uS in self.u[1:]:
						x      = np.zeros(self.J)
						### part one:
						x[(A==uA)&(B==uB)&(C==uC)&(S==uS)] =  1
						X.append(x)
		return np.matrix(X).T



	def get_design_main(self):
		X = []
		for u in self.u[1:]:
			x        = np.zeros(self.J)
			x[self.A==u] =  1
			X.append(x)
		return np.matrix( X ).T


	def get_design_main_nested(self, other):
		A,S  = other.A, self.A
		X    = []
		for uA in other.u:
			uS = np.unique(S[A==uA])
			for u in uS[1:]:
				x        = np.zeros(self.J)
				x[(A==uA) & (S==u)] =  1
				X.append(x)
		return np.matrix( X ).T
		
		
	def get_design_interaction_nested(self, other, another):
		A,B,S  = another.A, other.A, self.A
		XAB        = []
		for uA in np.unique(A):
			uS = np.unique(S[A==uA])
			
			for uB in other.u[1:]:
				for u in uS[1:]:
					x      = np.zeros(self.J)
					x[(A==uA)&(B==uB)&(S==u)] =  1
					XAB.append(x)
		return np.matrix(XAB).T
	





class FactorNested(Factor):
	def __init__(self, A, nestfactor):
		self.NEST       = nestfactor
		super(FactorNested, self).__init__(A)


	def _00_parse(self):
		self.J          = self.A.size
		self.u          = [np.unique(self.A[self.NEST.A==u])  for u in self.NEST.u]
		self.n          = [u.size for u in self.u]

	def _01_check_unbalanced(self):
		pass
		# nn  = [(self.A==uu).sum()   for uu in self.u]
		# self.balanced = np.all( nn==nn[0] )

	def get_Q(self):
		Q = []
		for u in self.u:
			for uu in u:
				q = np.matrix(np.diag(self.A==uu), dtype=float)
				Q.append(q)
		return Q
	
	
	def get_design_main(self):
		A,B   = self.NEST.A, self.A
		X     = []
		for uA,UB in zip(self.NEST.u, self.u):
			for uB in UB[1:]:
				x        = np.zeros(self.J)
				x[(A==uA) & (B==uB)] = 1
				X.append(x)
		return np.matrix( X ).T
		

	def get_design_interaction(self, other):
		A,B,S  = self.NEST.A, other.A, self.A
		X      = []
		for uA,US in zip(self.NEST.u, self.u):
			for uB in other.u[1:]:
				for uS in US[1:]:
					x      = np.zeros(self.J)
					x[(A==uA)&(B==uB)&(S==uS)] =  1
					X.append(x)
		return np.matrix(X).T


	def get_design_interaction_3way(self, other, another):
		A,B,C,S  = self.NEST.A, other.A, another.A, self.A
		X        = []
		for uA,US in zip(self.NEST.u, self.u):
			for uB in other.u[1:]:
				for uC in another.u[1:]:
					for uS in US[1:]:
						x      = np.zeros(self.J)
						x[(A==uA)&(B==uB)&(C==uC)&(S==uS)] =  1
						X.append(x)
		return np.matrix(X).T




class FactorNested2(Factor):
	def __init__(self, A, nestfactor):
		pass
		self.NEST0      = nestfactor.NEST
		self.NEST1      = nestfactor
		super(FactorNested2, self).__init__(A)


	def _00_parse(self):
		A,B,C           = self.NEST0.A, self.NEST1.A, self.A
		uA,UB           = self.NEST0.u, self.NEST1.u
		self.J          = self.A.size
		self.u          = [[np.unique(C[(A==uuA)&(B==uuB)]) for uuB in uB]  for uuA,uB in zip(uA, UB)]
		self.n          = [[uu.size for uu in u] for u in self.u]

	def _01_check_unbalanced(self):
		pass

	def get_Q(self):
		Q = []
		for u in self.u:
			for uu in u:
				for uuu in uu:
					q = np.matrix(np.diag(self.A==uuu), dtype=float)
					Q.append(q)
		return Q

	def get_design_main(self):
		A,B,C = self.NEST0.A, self.NEST1.A, self.A
		X     = []
		for uA,UB,UUC in zip(self.NEST0.u, self.NEST1.u, self.u):
			for uB,UC in zip(UB, UUC):
				for uC in UC[1:]:
					x        = np.zeros(self.J)
					x[(A==uA) & (B==uB) & (C==uC)] = 1
					X.append(x)
		return np.matrix( X ).T




class FactorNestedTwoWay(Factor):
	def __init__(self, SUBJ, A, B):
		self.NEST0      = A
		self.NEST1      = B
		super(FactorNestedTwoWay, self).__init__(SUBJ)


	def _00_parse(self):
		A,B,S           = self.NEST0.A, self.NEST1.A, self.A
		uA,uB           = self.NEST0.u, self.NEST1.u
		self.J          = self.A.size
		self.u          = [[np.unique( S[(A==uuA)&(B==uuB)] )  for uuB in uB] for uuA in uA]
		self.n          = [[uu.size for uu in u] for u in self.u]

	def _01_check_unbalanced(self):
		pass


	def get_Q(self):
		pass
		# Q = []
		# for u in self.u:
		# 	for uu in u:
		# 		for uuu in uu:
		# 			q = np.matrix(np.diag(self.A==uuu), dtype=float)
		# 			Q.append(q)
		# return Q


		
	def get_design_main(self):
		A,B,S    = self.NEST0.A, self.NEST1.A, self.A
		uA,uB,uS = np.unique(A), np.unique(B), np.unique(S)
		X        = []
		for uuA in uA:
			for uuB in uB:
				uS  = np.unique(S[(A==uuA) & (B==uuB)])
				for uuS in uS[1:]:
					x        = np.zeros(self.J)
					x[(A==uuA) & (B==uuB) & (S==uuS)] = 1
					X.append(x)
		return np.matrix( X ).T


	def get_design_interaction(self, other):
		A,B,C,S  = self.NEST0.A, self.NEST1.A, other.A, self.A
		X        = []
		for uA in self.NEST0.u:
			for uB in self.NEST1.u[1:]:
				for uC in other.u:
					uS = np.unique( S[(A==uA)&(B==uB)&(C==uC)] )
					for uuS in uS[1:]:
						x      = np.zeros(self.J)
						x[(A==uA)&(B==uB)&(C==uC)&(S==uuS)] =  1
						X.append(x)
		return np.matrix(X).T







# coding: utf-8

import os
import numpy as np
from .. import _base




class SimulatedPataky2015c(_base.DatasetRegress, _base.Dataset1D):
	def _set_values(self):
		self.cite     = 'Pataky, T. C., Vanrenterghem, J., & Robinson, M. A. (2015). Zero- vs. one-dimensional, parametric vs. non-parametric, and confidence interval vs. hypothesis testing procedures in one-dimensional biomechanical trajectory analysis. Journal of Biomechanics, 1–9. http://doi.org/10.1016/j.jbiomech.2015.02.051'
		self.datafile = os.path.join(_base.get_datafilepath(), 'simscalar_datasetC.npy')
		self.Y        = np.load(self.datafile)
		self.x        = np.arange(self.Y.shape[0])
		self.z        = None
		self.r        = None
		self.df       = None
		self.p        = None


class SpeedGRF(_base.DatasetRegress, _base.Dataset1D):
	def __init__(self, subj=0):
		self.subj     = int(subj)
		super(SpeedGRF, self).__init__()

	def _set_values(self):
		self.cite     = 'Pataky, T. C., Caravaggi, P., Savage, R., Parker, D., Goulermas, J., Sellers, W., & Crompton, R. (2008). New insights into the plantar pressure correlates of walking speed using pedobarographic statistical parametric mapping (pSPM). Journal of Biomechanics, 41(9), 1987–1994.'
		fnameY        = os.path.join(_base.get_datafilepath(), 'ex_grf_subj%03d.npy'%self.subj)
		fnameX        = os.path.join(_base.get_datafilepath(), 'ex_grf_speeds.npy')
		self.datafile = fnameY
		self.Y        = np.load(fnameY)
		self.x        = np.load(fnameX)[:,self.subj]
		self.z        = None
		self.r        = None
		self.df       = None
		self.p        = None








import os
import numpy as np
from matplotlib import pyplot
import spm1d



#first-level SPM analysis: (within-subject effects)
nSubj         = 10
BETA          = []  #regression slopes
for subj in range(nSubj):
	dataset   = spm1d.data.uv1d.regress.SpeedGRF(subj=subj)
	Y,x       = dataset.get_data()
	t         = spm1d.stats.regress(Y, x) #conduct linear regression
	BETA.append( t.beta[0] )  #retrieve the regression slope
BETA          = np.array(BETA)


#second-level SPM analysis:  (between-subject random effects)
alpha         = 0.05
t             = spm1d.stats.ttest(BETA)
ti            = t.inference(alpha, two_tailed=True)


#plot:
pyplot.close('all')
ti.plot()
ti.plot_threshold_label(fontsize=12)
ti.plot_p_values()
pyplot.xlabel('Time (% stance)', size=16)
pyplot.show()



import spm1d





#(0) Load dataset:
dataset      = spm1d.data.uv0d.anova3rm.SPM1D2x2x2()
# dataset      = spm1d.data.uv0d.anova3rm.SPM1D2x3x5()
# dataset      = spm1d.data.uv0d.anova3rm.SPM1D3x3x3()
y,A,B,C,SUBJ = dataset.get_data()
print dataset




#(1) Run ANOVA:
F = spm1d.stats.anova3rm(y, A, B, C, SUBJ, equal_var=True)
Fvalues = [f.z for f in F]
print Fvalues






from matplotlib import pyplot
import spm1d



#(0) Load data:
dataset      = spm1d.data.mv1d.hotellings_paired.Neptune1999kneekin()
YA,YB        = dataset.get_data()  #A:foot, B:speed
print dataset



#(1) Conduct test:
alpha        = 0.05
T2           = spm1d.stats.hotellings_paired(YA, YB)
T2i          = T2.inference(0.05)



#(2) Plot:
pyplot.close('all')
ax0     = pyplot.subplot(221)
ax1     = pyplot.subplot(222)
ax2     = pyplot.subplot(223)
ax3     = pyplot.subplot(224)
### plot SPM results:
h=ax0.plot(YA[:,:,0].T, 'k'); h[0].set_label('side-shuffle')
h=ax0.plot(YB[:,:,0].T, 'r'); h[0].set_label('v-cut')
ax1.plot(YA[:,:,1].T, 'k')
ax1.plot(YB[:,:,1].T, 'r')
ax2.plot(YA[:,:,2].T, 'k')
ax2.plot(YB[:,:,2].T, 'r')
T2i.plot(ax=ax3)
ax0.legend(fontsize=9)
pyplot.show()






import spm1d









#(0) Load dataset:
dataset  = spm1d.data.uv0d.anova1rm.Abdi2010()
# dataset  = spm1d.data.uv0d.anova1rm.Groceries()
# dataset  = spm1d.data.uv0d.anova1rm.Imacelebrity()
# dataset  = spm1d.data.uv0d.anova1rm.Southampton1rm()
y,A,SUBJ = dataset.get_data()
print dataset


#(1) Run ANOVA:
F = spm1d.stats.anova1rm(y, A, SUBJ, equal_var=True)
print F









import spm1d




#(0) Load data:
dataset   = spm1d.data.uv0d.anova3nested.SouthamptonNested3()
y,A,B,C   = dataset.get_data()
print dataset


#(1) Run ANOVA:
F = spm1d.stats.anova3nested(y, A, B, C)
Fvalues = [f.z for f in F]
DF = [f.df for f in F]
print Fvalues
print DF






import os
import numpy as np
from .. import _base



class _SPM1D_ANOVA2NESTED_DATASET(_base.DatasetANOVA2nested, _base.Dataset1D):
	def _set_values(self):
		self._set_datafile()
		Z             = np.load(self.datafile)
		self.Y,self.A,self.B = Z['Y'], Z['A'], Z['B']
		Z.close()


class SPM1D_ANOVA2NESTED_2x2(_SPM1D_ANOVA2NESTED_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2nested_2x2.npz')
class SPM1D_ANOVA2NESTED_2x3(_SPM1D_ANOVA2NESTED_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2nested_2x3.npz')

class SPM1D_ANOVA2NESTED_3x3(_SPM1D_ANOVA2NESTED_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2nested_3x3.npz')
class SPM1D_ANOVA2NESTED_3x4(_SPM1D_ANOVA2NESTED_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2nested_3x4.npz')
class SPM1D_ANOVA2NESTED_3x5(_SPM1D_ANOVA2NESTED_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2nested_3x5.npz')

class SPM1D_ANOVA2NESTED_4x4(_SPM1D_ANOVA2NESTED_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2nested_4x4.npz')
class SPM1D_ANOVA2NESTED_4x5(_SPM1D_ANOVA2NESTED_DATASET):
	def _set_datafile(self):
		self.datafile = os.path.join(_base.get_datafilepath(), 'spm1d_anova2nested_4x5.npz')